@@ -8,17 +8,17 @@
 #include <gtk/gtk.h>
 
 
-extern GtkWidget *AnimIconNew(gint width, gint height);
+extern GtkWidget *AnimIconNew(const gint width, const gint height);
 extern void AnimIconSetFromXPMDataList(
-	GtkWidget *w, GList *data_list, gulong interval
+	GtkWidget *w, GList *data_list, const gulong interval
 );
 extern void AnimIconSetColorBG(GtkWidget *w, const GdkColor *c);
-extern void AnimIconSetRepeating(GtkWidget *w, gboolean repeating);
+extern void AnimIconSetRepeating(GtkWidget *w, const gboolean repeating);
 
 extern gint AnimIconFrames(GtkWidget *w);
 extern gint AnimIconFrame(GtkWidget *w);
 extern gboolean AnimIconIsPlaying(GtkWidget *w);
-extern void AnimIconSeek(GtkWidget *w, gint frame);
+extern void AnimIconSeek(GtkWidget *w, const gint frame);
 extern void AnimIconPause(GtkWidget *w);
 extern void AnimIconPlay(GtkWidget *w);
 
