@@ -7,20 +7,24 @@
 #include <fcntl.h>
 #include <dirent.h>
 #include <sys/stat.h>
-#include <unistd.h>
+#ifdef _WIN32
+# include <windows.h>
+#else
+# include <unistd.h>
+#endif
 
 #if defined(HAVE_CLAMAV)
-
 # include <clamav.h>
+# if !defined(USE_CLAMAV_88) && !defined(CL_DB_STDOPT)
+#  error CL_DB_STDOPT was not defined, you are most likely using an older version of ClamAV, see comment about USE_CLAMAV_88 in the Makefile
+# endif
 #else
 #warning No antivirus library defined, this module will not perform any virus scanning.
 #endif
 
 #include <glib.h>
 
-#include "../include/fio.h"
 #include "../include/disk.h"
-#include "../include/prochandle.h"
 
 #include "avscanop.h"
 #include "config.h"
@@ -42,6 +46,9 @@ static void PRINT_CLEAN(const gchar *path);
 static void PRINT_INFECTED(const gchar *path, const gchar *virus);
 static void PRINT_PROBLEM(const gchar *path, const gchar *problem);
 
+static gboolean AVScanIsExecutable(
+	const gchar *path, const mode_t m
+);
 static gchar *AVScanCopyShortenString(const gchar *s, const gint m);
 
 
@@ -49,7 +56,11 @@ static gchar *AVScanCopyShortenString(const gchar *s, const gint m);
 typedef struct {
 	gchar		*database_path;		/* Database file or directory */
  	gchar		*database_dir;		/* Database directory */
+#if defined(USE_CLAMAV_88)
 	struct cl_node	*database;		/* Database handle */
+#else	/* ClamAV 90.x or 91.x */
+	struct cl_engine	*database;	/* Database handle */
+#endif
 	gint		database_entries;	/* Entries in the database */
 	struct cl_stat	database_stat;		/* Database stats, for
 						 * checking database changes */
@@ -179,6 +190,53 @@ static void PRINT_PROBLEM(const gchar *path, const gchar *problem)
 }
 
 
+/*
+ *	Checks if the object is an executable.
+ */
+static gboolean AVScanIsExecutable(
+	const gchar *path, const mode_t m
+)
+{
+#if defined(_WIN32)
+	const gchar *ext, *name;
+
+	if(path == NULL)
+	    return(FALSE);
+
+	/* Get the name */
+	name = g_basename(path);
+	if(name == NULL)
+	    name = path;
+
+	/* Get the extension */
+	ext = (const gchar *)strrchr((const char *)name, '.');
+	if(ext == NULL)
+	    return(FALSE);
+
+	ext++;		/* Seek past the '.' deliminator */
+
+	if(!g_strcasecmp(ext, "bat") ||
+	   !g_strcasecmp(ext, "com") ||
+	   !g_strcasecmp(ext, "exe") ||
+	   !g_strcasecmp(ext, "bin") ||
+	   !g_strcasecmp(ext, "dll") ||
+	   !g_strcasecmp(ext, "lib") ||
+	   !g_strcasecmp(ext, "so") ||
+	   !g_strcasecmp(ext, "o") ||
+	   !g_strcasecmp(ext, "a")
+	)
+	    return(TRUE);
+	else
+	    return(FALSE);
+#else
+	if((m & S_IXUSR) || (m & S_IXGRP) || (m & S_IXOTH))
+	    return(TRUE);
+	else
+	    return(FALSE);
+#endif
+}
+
+
 /*
  *	Returns a copy of the string that is no longer than max
  *	characters with appropriate shortened notation where 
@@ -195,7 +253,7 @@ static gchar *AVScanCopyShortenString(const gchar *s, const gint m)
 	if((len > m) && (m > 3))
 	{
 	    /* Need to shorten string */
-	    gint i = len - m + 3;
+	    const gint i = len - m + 3;
 
 	    return(g_strdup_printf(
 		"...%s", (const gchar *)(s + i)
@@ -260,13 +318,7 @@ static ClamAVData *AVScanOPClamAVInit(
 	if(stat((const char *)d->database_path, &stat_buf))
 	{
 	    const gint error_code = (gint)errno;
-	    gchar *error_msg = STRDUP(g_strerror(error_code));
-	    if(error_msg != NULL)
-	    {
-		*error_msg = (gchar)toupper((int)*error_msg);
-		PRINT_PROBLEM(d->database_path, error_msg);
-		g_free(error_msg);
-	    }
+	    PRINT_PROBLEM(d->database_path, g_strerror(error_code));
 	    g_free(d->database_path);
 	    g_free(d);
 	    *status = 50;
@@ -297,10 +349,18 @@ static ClamAVData *AVScanOPClamAVInit(
 		return(NULL);
 	    }
 
+#if defined(USE_CLAMAV_88)
 	    status2 = cl_loaddbdir(
 		d->database_path,
 		&d->database, &d->database_entries
 	    );
+#else	/* ClamAV 90.x or 91.x */
+	    status2 = cl_load(
+		d->database_path,
+		&d->database, &d->database_entries,
+		CL_DB_STDOPT
+	    );
+#endif
 	}
 	else
 	{
@@ -314,10 +374,18 @@ static ClamAVData *AVScanOPClamAVInit(
 		return(NULL);
 	    }
 
+#if defined(USE_CLAMAV_88)
 	    status2 = cl_loaddb(
 		d->database_path,
 		&d->database, &d->database_entries
 	    );
+#else	/* ClamAV 90.x or 91.x */
+	    status2 = cl_load(
+		d->database_path,
+		&d->database, &d->database_entries,
+		CL_DB_STDOPT
+	    );
+#endif
 	}
 	if((status2 != 0) || (d->database == NULL) || (d->database_entries <= 0))
 	{
@@ -428,13 +496,7 @@ static gint AVScanOPClamAVScanFile(
 	if(fd < 0)
 	{
 	    const gint error_code = (gint)errno;
-	    gchar *error_msg = STRDUP(g_strerror(error_code));
-	    if(error_msg != NULL)
-	    {
-		*error_msg = (gchar)toupper((int)*error_msg);
-		PRINT_PROBLEM(path, error_msg);
-		g_free(error_msg);
-	    }
+	    PRINT_PROBLEM(path, g_strerror(error_code));
 	    return(54);
 	}
 
@@ -499,21 +561,33 @@ static gint AVScanOPClamAVScanFile(
 	d->files_scanned++;
 	d->size_scanned_blocks += (gulong)scanned_blocks;
 
+#if defined(USE_CLAMAV_88)
 	PRINT_SCANNED(
 	    d->files_scanned,
-#ifdef CL_COUNT_PRECISION
+# ifdef CL_COUNT_PRECISION
 	    (gulong)(scanned_blocks *
 		((gfloat)CL_COUNT_PRECISION / 1024.0f)
 	    ),
 	    (gulong)((gfloat)d->size_scanned_blocks *
 		((gfloat)CL_COUNT_PRECISION / 1024.0f)
 	    )
-#else     
-#warning CL_COUNT_PRECISION not defined, assuming old ClamAV 1:1 precision.
+# else     
+# warning CL_COUNT_PRECISION not defined, assuming old ClamAV 1:1 precision.
 	    (gulong)scanned_blocks * 1l,
 	    d->size_scanned_blocks * 1l
-#endif
+# endif	/* CL_COUNT_PRECISION */
 	);
+#else	/* ClamAV 90.x or 91.x */
+	PRINT_SCANNED(
+	    d->files_scanned,
+	    (gulong)(scanned_blocks *
+		((gfloat)CL_COUNT_PRECISION / 1024.0f)
+	    ),
+	    (gulong)((gfloat)d->size_scanned_blocks *
+		((gfloat)CL_COUNT_PRECISION / 1024.0f)
+	    )
+	);
+#endif
 
 	/* Check the scan results */
 
@@ -551,84 +625,146 @@ static gint AVScanOPClamAVScanFile(
 	    }
 	    switch(status)
 	    {
+#ifdef CL_EMAXREC
 	      case CL_EMAXREC:
 		status = 57;
 		break;
+#endif
+#ifdef CL_EMAXSIZE
 	      case CL_EMAXSIZE:
 		status = 57;
 		break;
+#endif
+#ifdef CL_EMAXFILES
 	      case CL_EMAXFILES:
 		status = 57;
 		break;
+#endif
+#ifdef CL_ERAR
 	      case CL_ERAR:
 		status = 59;
 		break;
+#endif
+#ifdef CL_EZIP
 	      case CL_EZIP:
 		status = 59;
 		break;
+#endif
+#ifdef CL_EMALFZIP
 	      case CL_EMALFZIP:
 		status = 51;
 		break;
+#endif
+#ifdef CL_EGZIP
 	      case CL_EGZIP:
 		status = 59;
 		break;
+#endif
+#ifdef CL_EBZIP
 	      case CL_EBZIP:
 		status = 59;
 		break;
+#endif
+#ifdef CL_EOLE2
 	      case CL_EOLE2:
 		status = 59;
 		break;
+#endif
+#ifdef CL_EMSCOMP
 	      case CL_EMSCOMP:
 		status = 59;
 		break;
+#endif
+#ifdef CL_EMSCAB
 	      case CL_EMSCAB:
 		status = 59;
 		break;
+#endif
+#ifdef CL_EACCES
 	      case CL_EACCES:
 		status = 55;
 		break;
+#endif
+#ifdef CL_ENULLARG
 	      case CL_ENULLARG:
 		status = 2;
 		break;
+#endif
+#ifdef CL_ETMPFILE
 	      case CL_ETMPFILE:
 		status = 3;
 		break;
+#endif
+#ifdef CL_EFSYNC
 	      case CL_EFSYNC:
 		status = 3;
 		break;
+#endif
+#ifdef CL_EMEM
 	      case CL_EMEM:
 		status = 3;
 		break;
+#endif
+#ifdef CL_EOPEN
 	      case CL_EOPEN:
 		status = 54;
 		break;
+#endif
+#ifdef CL_EMALFDB
 	      case CL_EMALFDB:
 		status = 50;
 		break;
+#endif
+#ifdef CL_EPATSHORT
 	      case CL_EPATSHORT:
 		status = 2;
 		break;
+#endif
+#ifdef CL_ETMPDIR
 	      case CL_ETMPDIR:
 		status = 3;
 		break;
+#endif
+#ifdef CL_ECVD
 	      case CL_ECVD:
 		status = 50;
 		break;
+#endif
+#ifdef CL_ECVDEXTR
 	      case CL_ECVDEXTR:
 		status = 50;
 		break;
+#endif
+#ifdef CL_EMD5
 	      case CL_EMD5:
 		status = 50;
 		break;
+#endif
+#ifdef CL_EDSIG
 	      case CL_EDSIG:
 		status = 50;
 		break;
+#endif
+#ifdef CL_EIO
 	      case CL_EIO:
 		status = 3;
 		break;
+#endif
+#ifdef CL_EFORMAT
 	      case CL_EFORMAT:
 		status = 51;
 		break;
+#endif
+#ifdef ESUPPORT
+	      case ESUPPORT:
+		status = 60;
+		break;
+#endif
+#ifdef ELOCKDB
+	      case ELOCKDB:
+		status = 61;
+		break;
+#endif
 	      default:
 		status = 1;
 		break;
@@ -666,13 +802,7 @@ static gint AVScanOPClamAVScanDir(
 	if(dp == NULL)
 	{
 	    const gint error_code = (gint)errno;
-	    gchar *error_msg = STRDUP(g_strerror(error_code));
-	    if(error_msg != NULL)
-	    {
-		*error_msg = (gchar)toupper((int)*error_msg);
-		PRINT_PROBLEM(path, error_msg);
-		g_free(error_msg);
-	    }
+	    PRINT_PROBLEM(path, g_strerror(error_code));
 	    return(53);
 	}
 
@@ -722,13 +852,7 @@ static gint AVScanOPClamAVScanDir(
 		if(stat((const char *)d->database_path, &stat_buf))
 		{
 		    const gint error_code = (gint)errno;
-		    gchar *error_msg = STRDUP(g_strerror(error_code));
-		    if(error_msg != NULL)
-		    {
-			*error_msg = (gchar)toupper((int)*error_msg);
-			PRINT_PROBLEM(d->database_path, error_msg);
-			g_free(error_msg);
-		    }
+		    PRINT_PROBLEM(d->database_path, g_strerror(error_code));
 		    status = 50;
 		    break;
 		}
@@ -738,15 +862,35 @@ static gint AVScanOPClamAVScanDir(
 #else
 		if(FALSE)
 #endif
+		{
+#if defined(USE_CLAMAV_88)
 		    status2 = cl_loaddbdir(
 			d->database_path,
 			&d->database, &d->database_entries
 		    );
+#else	/* ClamAV 90.x or 91.x */
+		    status2 = cl_load(
+			d->database_path,
+			&d->database, &d->database_entries,
+			CL_DB_STDOPT
+		    );
+#endif
+		}
 		else
+		{
+#if defined(USE_CLAMAV_88)
 		    status2 = cl_loaddb(
 			d->database_path,
 			&d->database, &d->database_entries
 		    );
+#else	/* ClamAV 90.x or 91.x */
+		    status2 = cl_load(
+			d->database_path,
+			&d->database, &d->database_entries,
+			CL_DB_STDOPT
+		    );
+#endif
+		}
 		if((status2 != 0) || (d->database == NULL) ||
 		   (d->database_entries <= 0)
 		)
@@ -817,9 +961,12 @@ static gint AVScanOPClamAVScanDir(
 		continue;
 
 	    /* Get the full path to the child object */
-	    child = STRDUP(PrefixPaths(
-		(const char *)path, (const char *)s
-	    ));
+	    child = g_strconcat(
+		path,
+		G_DIR_SEPARATOR_S,
+		s,
+		NULL
+	    );
 	    if(child == NULL)
 	    {
 		if(status == 0)
@@ -831,13 +978,7 @@ static gint AVScanOPClamAVScanDir(
 	    if(lstat((const char *)child, &stat_buf))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *error_msg = STRDUP(g_strerror(error_code));
-		if(error_msg != NULL)
-		{
-		    *error_msg = (gchar)toupper((int)*error_msg);
-		    PRINT_PROBLEM(child, error_msg);
-		    g_free(error_msg);
-		}
+		PRINT_PROBLEM(child, g_strerror(error_code));
 		g_free(child);
 		if(status == 0)
 		    status = 56;
@@ -865,7 +1006,7 @@ static gint AVScanOPClamAVScanDir(
 		/* Get the destination value and check if it is a
 		 * reference to the current or parent directory
 		 */
-		dest = (gchar *)GetAllocLinkDest(child);
+		dest = (gchar *)GetAllocLinkDest((const char *)child);
 		if(dest != NULL)
 		{
 		    if(!strcmp((const char *)dest, ".") ||
@@ -885,13 +1026,7 @@ static gint AVScanOPClamAVScanDir(
 	    if(stat((const char *)child, &stat_buf))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *error_msg = STRDUP(g_strerror(error_code));
-		if(error_msg != NULL)
-		{
-		    *error_msg = (gchar)toupper((int)*error_msg);
-		    PRINT_PROBLEM(child, error_msg);
-		    g_free(error_msg);
-		}
+		PRINT_PROBLEM(child, g_strerror(error_code));
 		g_free(child);
 		if(status == 0)
 		    status = 56;
@@ -900,34 +1035,44 @@ static gint AVScanOPClamAVScanDir(
 
 	    m = stat_buf.st_mode;
 
-	    /* Directory and recursive? */
+	    /* Directory */
 #ifdef S_ISDIR
-	    if(S_ISDIR(m) && recursive)
+	    if(S_ISDIR(m))
 #else
 	    if(FALSE)
 #endif
 	    {
-		status2 = AVScanOPClamAVScanDir(
-		    d, child,
-		    recursive, executables_only, ignore_links,
-		    reload, stop_count,
-		    scanner_opts
-		);
-		if((status2 != 0) && (status == 0))
-		    status = status2;
+		if(recursive)
+		{
+		    status2 = AVScanOPClamAVScanDir(
+			d, child,
+			recursive, executables_only, ignore_links,
+			reload, stop_count,
+			scanner_opts
+		    );
+		    if((status2 != 0) && (status == 0))
+			status = status2;
+		}
 	    }
-	    /* Executables only or all? */
-	    else if(executables_only ?
-	((m & S_IXUSR) || (m & S_IXGRP) || (m & S_IXOTH)) : TRUE
-	    )
+	    /* Regular File */
+#ifdef S_ISREG
+	    else if(S_ISREG(m))
+#else
+	    else if(TRUE)
+#endif
 	    {
-		status2 = AVScanOPClamAVScanFile(
-		    d, child,
-		    stop_count,
-		    scanner_opts
-		);
-		if((status2 != 0) && (status == 0))
-		    status = status2;
+		if(executables_only ?
+		    AVScanIsExecutable(child, m) : TRUE
+		)
+		{
+		    status2 = AVScanOPClamAVScanFile(
+			d, child,
+			stop_count,
+			scanner_opts
+		    );
+		    if((status2 != 0) && (status == 0))
+			status = status2;
+		}
 	    }
 
 	    g_free(child);
@@ -991,9 +1136,12 @@ static gulong AVScanOPCalculateTotalFilesInDirIterate(
 		continue;
 
 	    /* Get the full path to the child object */
-	    child = STRDUP(PrefixPaths(
-		(const char *)path, (const char *)s   
-	    ));
+	    child = g_strconcat(
+		path,
+		G_DIR_SEPARATOR_S,
+		s,
+		NULL
+	    );
 	    if(child == NULL)
 		continue;
 
@@ -1053,27 +1201,31 @@ static gulong AVScanOPCalculateTotalFilesInDirIterate(
 
 	    m = stat_buf.st_mode;
 
-	    /* Is the destination a directory and should we recurse
-	     * into it?
-	     */
+	    /* Directory */
 #ifdef S_ISDIR
-	    if(S_ISDIR(m) && recursive)
+	    if(S_ISDIR(m))
 #else
 	    if(FALSE)
 #endif
 	    {
-		subtotal += AVScanOPCalculateTotalFilesInDirIterate(
-		    child,
-		    recursive, executables_only, ignore_links,
-		    stop_count
-		);
+		if(recursive)
+		    subtotal += AVScanOPCalculateTotalFilesInDirIterate(
+			child,
+			recursive, executables_only, ignore_links,
+			stop_count
+		    );
 	    }
-	    /* Executable only or all? */
-	    else if(executables_only ?
-	((m & S_IXUSR) || (m & S_IXGRP) || (m & S_IXOTH)) : TRUE
-	    )
+	    /* Regular File */
+#ifdef S_ISREG
+	    else if(S_ISREG(m))
+#else
+	    else if(TRUE)
+#endif
 	    {
-		subtotal++;
+		if(executables_only ?
+		    AVScanIsExecutable(child, m) : TRUE
+		)
+		    subtotal++;
 	    }
 
 	    g_free(child);
@@ -1128,7 +1280,7 @@ static gulong AVScanOPCalculateTotalFiles(
 
 	    m = stat_buf.st_mode;
 
-	    /* Link? */
+	    /* Link */
 #ifdef S_ISLNK
 	    if(S_ISLNK(m))
 #else
@@ -1169,7 +1321,7 @@ static gulong AVScanOPCalculateTotalFiles(
 		    total++;
 		}
 	    }
-	    /* Directory? */
+	    /* Directory */
 #ifdef S_ISDIR
 	    else if(S_ISDIR(m))
 #else
@@ -1182,7 +1334,7 @@ static gulong AVScanOPCalculateTotalFiles(
 		    stop_count
 		);
 	    }
-	    /* File? */
+	    /* Regular File */
 #ifdef S_ISREG
 	    else if(S_ISREG(m))
 #else
@@ -1252,6 +1404,9 @@ gchar *AVScanGetAVEngineNameVersionString(void)
  *	59	Scanner subsystem or plugin unable to scan an input
  *		file.
  *
+ *	60	Unsupported file format.
+ *	61	Unable to lock the virus database directory.
+ *
  *	99	No virus scanner support compiled.
  */
 gint AVScanOP(
@@ -1315,15 +1470,8 @@ gint AVScanOP(
 		/* Get this object's local stats */
 		if(lstat((const char *)path, &stat_buf))
 		{
-		    /* Unable to get this object's local stats */
 		    const gint error_code = (gint)errno;
-		    gchar *error_msg = STRDUP(g_strerror(error_code));
-		    if(error_msg != NULL)
-		    {
-			*error_msg = (gchar)toupper((int)*error_msg);
-			PRINT_PROBLEM(path, error_msg);
-			g_free(error_msg);
-		    }
+		    PRINT_PROBLEM(path, g_strerror(error_code));
 		    if(status == 0)
 			status = 56;
 		    continue;
@@ -1331,7 +1479,7 @@ gint AVScanOP(
 
 		m = stat_buf.st_mode;
 
-		/* Link? */
+		/* Link */
 #ifdef S_ISLNK
 		if(S_ISLNK(m))
 #else
@@ -1351,15 +1499,8 @@ gint AVScanOP(
 		    /* Get this link's destination stats */
 		    if(stat((const char *)path, &stat_buf))
 		    {
-			/* Unable to get this link's destination stats */
 			const gint error_code = (gint)errno;
-			gchar *error_msg = STRDUP(g_strerror(error_code));
-			if(error_msg != NULL)
-			{
-			    *error_msg = (gchar)toupper((int)*error_msg);
-			    PRINT_PROBLEM(path, error_msg);
-			    g_free(error_msg);
-			}
+			PRINT_PROBLEM(path, g_strerror(error_code));
 			if(status == 0)
 			    status = 56;
 			continue;
@@ -1390,7 +1531,7 @@ gint AVScanOP(
 			);
 		    }
 		}
-		/* Directory? */
+		/* Directory */
 #ifdef S_ISDIR
 		else if(S_ISDIR(m))
 #else
@@ -1402,7 +1543,7 @@ gint AVScanOP(
 			reload, stop_count,
 			scanner_opts
 		    );
-		/* File? */
+		/* Regular File */
 #ifdef S_ISREG
 		else if(S_ISREG(m))
 #else
