@@ -2,6 +2,7 @@
 #define AVSCANOP_H
 
 #include <glib.h>
+#include "scanner_opts.h"
 
 
 extern gchar *AVScanGetAVEngineNameVersionString(void);
@@ -12,7 +13,8 @@ extern gint AVScanOP(
 	const gboolean recursive,
 	const gboolean executables_only,
 	const gboolean ignore_links,
-	gint *stop_count
+	gboolean *reload, gint *stop_count,
+	scanner_opts_struct *scanner_opts
 );
 
 
