@@ -23,6 +23,17 @@ void CFGAccelkeyListDelete(cfg_accelkey_list_struct *accelkey_list);
 cfg_style_struct *CFGStyleNew(void);
 void CFGStyleDelete(cfg_style_struct *style);
 
+cfg_menu_item_struct *CFGMenuItemNew(
+	const gchar *label,
+	const gchar *command,
+	const gchar *icon_file,
+	const gchar *description,
+	const gchar *ext_data
+);
+void CFGMenuItemDelete(cfg_menu_item_struct *mi);
+cfg_menu_struct *CFGMenuNew(GList *glist);
+void CFGMenuDelete(cfg_menu_struct *m);
+
 
 /* Cfg Item */
 cfg_item_struct *CFGItemNew(
@@ -80,6 +91,10 @@ cfg_accelkey_list_struct *CFGItemListGetValueAccelkeyList(
 cfg_style_struct *CFGItemListGetValueStyle(
 	const cfg_item_struct *list, const gchar *parameter
 );
+cfg_menu_struct *CFGItemListGetValueMenu(
+	const cfg_item_struct *list, const gchar *parameter
+);
+
 
 /* Cfg Item Value Setting */
 void CFGItemListSetValue(
@@ -126,6 +141,10 @@ void CFGItemListSetValueStyle(
 	cfg_item_struct *list, const gchar *parameter,
 	const cfg_style_struct *value, gboolean create_as_needed
 );
+void CFGItemListSetValueMenu(
+	cfg_item_struct *list, const gchar *parameter,
+	const cfg_menu_struct *value, gboolean create_as_needed
+);
 
 
 #ifndef g_memcpy
@@ -260,17 +279,17 @@ cfg_accelkey_list_struct *CFGAccelkeyListNew(GList *glist)
 	while(glist != NULL)
 	{
 	    akey = CFG_ACCELKEY(glist->data);
-	    if(akey == NULL)
-		continue;
-
-	    akey_list->list = g_list_append(
-		akey_list->list,
-		CFGAccelkeyNew(
-		    akey->opid,
-		    akey->key,
-		    akey->modifiers
-		)
-	    );
+	    if(akey != NULL)
+	    {
+		akey_list->list = g_list_append(
+		    akey_list->list,
+		    CFGAccelkeyNew(
+			akey->opid,
+			akey->key,
+			akey->modifiers
+		    )
+		);
+	    }
 	    glist = g_list_next(glist);
 	}
 
@@ -328,6 +347,99 @@ void CFGStyleDelete(cfg_style_struct *style)
 }
 
 
+/*
+ *	Creates a new Cfg Menu Item.
+ */
+cfg_menu_item_struct *CFGMenuItemNew(
+	const gchar *label,
+	const gchar *command,
+	const gchar *icon_file,
+	const gchar *description,
+	const gchar *ext_data
+)
+{
+	cfg_menu_item_struct *mi = CFG_MENU_ITEM(g_malloc0(
+	    sizeof(cfg_menu_item_struct)
+	));
+	if(mi == NULL)
+	    return(NULL);
+
+	mi->label = STRDUP(label);
+	mi->command = STRDUP(command);
+	mi->icon_file = STRDUP(icon_file);
+	mi->description = STRDUP(description);
+	mi->ext_data = STRDUP(ext_data);
+
+	return(mi);
+}
+
+/*
+ *	Deletes the Cfg Menu Item.
+ */
+void CFGMenuItemDelete(cfg_menu_item_struct *mi)
+{
+	if(mi == NULL)
+	    return;
+
+	g_free(mi->label);
+	g_free(mi->command);
+	g_free(mi->icon_file);
+	g_free(mi->description);
+	g_free(mi);
+}
+
+/*
+ *	Creates a new Cfg Menu.
+ */
+cfg_menu_struct *CFGMenuNew(GList *glist)
+{
+	cfg_menu_item_struct *mi;
+	cfg_menu_struct *m = CFG_MENU(g_malloc0(
+	    sizeof(cfg_menu_struct)
+	));
+	if(m == NULL)
+	    return(NULL);
+
+	while(glist != NULL)
+	{
+	    mi = CFG_MENU_ITEM(glist->data);
+	    if(mi != NULL)
+	    {
+		m->list = g_list_append(
+		    m->list,
+		    CFGMenuItemNew(
+			mi->label,
+			mi->command,
+			mi->icon_file,
+			mi->description,
+			mi->ext_data
+		    )
+		);
+	    }
+	    glist = g_list_next(glist);
+	}
+
+	return(m);
+}
+
+/*
+ *	Deletes the Cfg Menu.
+ */
+void CFGMenuDelete(cfg_menu_struct *m)
+{
+	if(m == NULL)
+	    return;
+
+	if(m->list != NULL)
+	{
+	    g_list_foreach(m->list, (GFunc)CFGMenuItemDelete, NULL);
+	    g_list_free(m->list);
+	}
+
+	g_free(m);
+}
+
+
 /*
  *	Creates a new Cfg Item.
  */
@@ -359,12 +471,12 @@ gpointer CFGItemGetValue(
 	{
 	    if(type_rtn != NULL)
 		*type_rtn = ci->type;
-	    return((gpointer)ci->value);
+	    return(ci->value);
 	}
 	else
 	{
 	    if(type_rtn != NULL)
-		*type_rtn = -1;
+		*type_rtn = CFG_ITEM_TYPE_NONE;
 	    return(NULL);
 	}
 }
@@ -394,11 +506,12 @@ void CFGItemSetValue(cfg_item_struct *ci, gconstpointer value)
 	cfg_color_struct *ptr_color;
 	cfg_accelkey_list_struct *ptr_ak_list;
 	cfg_style_struct *ptr_style;
+	cfg_menu_struct *ptr_menu;
 
 	if(ci == NULL)
 	    return;
 
-	/* Delete existing value on the cfg item */
+	/* Delete the existing value on the cfg item */
 	CFGItemResetValue(ci);
 
 	/* No value given? */
@@ -576,6 +689,22 @@ void CFGItemSetValue(cfg_item_struct *ci, gconstpointer value)
 	    }
 	    ci->value = ptr_style;
 	    break;
+
+	  case CFG_ITEM_TYPE_MENU:
+	    size = sizeof(cfg_menu_struct);
+	    if(size > 0)
+	    {
+		const cfg_menu_struct *src_menu = CFG_MENU(value);
+		ptr_menu = CFGMenuNew(
+		    src_menu->list
+		);
+	    }
+	    else
+	    {
+		ptr_menu = NULL;
+	    }
+	    ci->value = ptr_menu;
+	    break;
 	}
 }
 
@@ -630,6 +759,10 @@ void CFGItemResetValue(cfg_item_struct *ci)
 	    CFGStyleDelete(CFG_STYLE(ci->value));
 	    ci->value = NULL;
 	    break;
+	  case CFG_ITEM_TYPE_MENU:
+	    CFGMenuDelete(CFG_MENU(ci->value));
+	    ci->value = NULL;
+	    break;
 	}
 
 	/* Delete value in case it was not deleted above and reset it to
@@ -648,14 +781,14 @@ void CFGItemReset(cfg_item_struct *ci)
 	if(ci == NULL)
 	    return;
 
-	/* Delete value */
+	/* Delete the value */
 	CFGItemResetValue(ci);
 
-	/* Delete parameter */
+	/* Delete the parameter */
 	g_free(ci->parameter);
 	ci->parameter = NULL;
 
-	/* Reset type */
+	/* Reset the type */
 	ci->type = CFG_ITEM_TYPE_NONE;
 }
 
@@ -715,10 +848,10 @@ cfg_item_struct *CFGItemListCopyList(
 		break;
 	    }
 
-	    /* Get pointer to new target cfg item */
+	    /* Get the pointer to new target cfg item */
 	    tar_cfg_item = &tar_list[i];
 
-	    /* Reset target cfg item */
+	    /* Reset the target cfg item */
 	    memset(tar_cfg_item, 0x00, sizeof(cfg_item_struct));
 
 
@@ -813,6 +946,13 @@ cfg_item_struct *CFGItemListCopyList(
 			FALSE
 		    );
 		    break;
+		  case CFG_ITEM_TYPE_MENU:
+		    CFGItemListSetValueMenu(
+			tar_cfg_item, tar_cfg_item->parameter,
+			CFG_MENU(src_cfg_item->value),
+			FALSE
+		    );
+		    break;
 		}
 	    }
 
@@ -845,7 +985,6 @@ void CFGItemListDeleteList(cfg_item_struct *list)
 	gint i;
 	cfg_item_struct *ci;
 
-
 	if(list == NULL)
 	    return;
 
@@ -859,7 +998,7 @@ void CFGItemListDeleteList(cfg_item_struct *list)
 	      (ci->value != NULL)
 	)
 	{
-	    /* Delete cfg item's parameter and value */
+	    /* Delete the cfg item's parameter and value */
 	    CFGItemReset(ci);
 
 	    /* Go on to the next cfg item */
@@ -878,31 +1017,30 @@ void CFGItemListDeleteList(cfg_item_struct *list)
  *
  *	The last item in the sequential list needs to be all 0.
  *
- *	Returns -1 on failed match or error, otherwise returns
- *	the index of the valid item.
+ *	Returns the index of the matched item or negative on error.
  */
 gint CFGItemListMatchParameter(
 	const cfg_item_struct *list, const gchar *parameter
 )
 {
 	gint i;
-	const cfg_item_struct *ci;
-
+	const cfg_item_struct *item;
 
 	if((list == NULL) || STRISEMPTY(parameter))
-	    return(-1);
+	    return(-2);
 
 	i = 0;
-	ci = &list[i];
-	while((ci->type != CFG_ITEM_TYPE_NONE) &&
-	      (ci->parameter != NULL)
-	)
+	item = &list[i];
+	while(item->type != CFG_ITEM_TYPE_NONE)
 	{
-	    if(!g_strcasecmp(ci->parameter, parameter))
-		return(i);
+	    if(item->parameter != NULL)
+	    {
+		if(!g_strcasecmp(item->parameter, parameter))
+		    return(i);
+	    }
 
 	    i++;
-	    ci = &list[i];
+	    item = &list[i];
 	}
 
 	return(-1);
@@ -923,16 +1061,20 @@ gpointer CFGItemListMatchGetValue(
 {
 	gint i;
 
-	if(type_rtn != NULL)
-	    *type_rtn = -1;
-
-	/* Check if given parameter matches the parameter of a cfg item
-	 * in the list
+	/* Check if the specified parameter matches the parameter of a
+	 * cfg item in the list
 	 */
 	i = CFGItemListMatchParameter(list, parameter);
-	return((i > -1) ?
-	    CFGItemGetValue(&list[i], type_rtn) : NULL
-	);
+	if(i < 0)
+	{
+	    /* No such parameter */
+	    if(type_rtn != NULL)
+		*type_rtn = CFG_ITEM_TYPE_NONE;
+	    return(NULL);
+	}
+
+	/* Return the parameter's value and type */
+	return(CFGItemGetValue(&list[i], type_rtn));
 }
 
 /*
@@ -964,7 +1106,6 @@ gint CFGItemListGetValueI(
 	gint32 *ptr_32;
 	guint32 *ptr_u32;
 
-
 	/* Match by parameter */
 	ci_num = CFGItemListMatchParameter(list, parameter);
 	if(ci_num < 0)
@@ -1435,6 +1576,37 @@ cfg_style_struct *CFGItemListGetValueStyle(
 	    return(CFG_STYLE(ci_ptr->value));
 }
 
+/*
+ *	Returns the menu value of the Cfg Item who matches
+ *	the given parameter found in the given Cfg Items List.
+ *
+ *	Only Cfg Items of the following types will be matched:
+ *
+ *		CFG_ITEM_TYPE_MENU
+ */
+cfg_menu_struct *CFGItemListGetValueMenu(
+	const cfg_item_struct *list, const gchar *parameter
+)
+{
+	gint ci_num;
+	const cfg_item_struct *ci_ptr;
+
+	/* Match by parameter */
+	ci_num = CFGItemListMatchParameter(list, parameter);
+	if(ci_num < 0)
+	    return(NULL);
+	else
+	    ci_ptr = &list[ci_num];
+
+	/* Not the correct type or no value? */
+	if((ci_ptr->type != CFG_ITEM_TYPE_MENU) ||
+	   (ci_ptr->value == NULL)
+	)
+	    return(NULL);
+	else
+	    return(CFG_MENU(ci_ptr->value));
+}
+
 
 /*
  *      Sets the given value to the sequential list item found match
@@ -1934,3 +2106,38 @@ void CFGItemListSetValueStyle(
 /* TODO */
 	}
 }
+
+/*
+ *	Sets value as a cfg_menu_struct, the parameter must
+ *	match to an item who's type matches any of the following:
+ *
+ *		CFG_ITEM_TYPE_MENU
+ */
+void CFGItemListSetValueMenu(
+	cfg_item_struct *list, const gchar *parameter,
+	const cfg_menu_struct *value, gboolean create_as_needed
+)
+{
+	gint i;
+
+	if(list == NULL)
+	    return;
+
+	/* First try to match an existing configuration item in the
+	 * given list of cfg items
+	 */
+	i = CFGItemListMatchParameter(list, parameter);
+	if(i > -1)
+	{
+	    if((&list[i])->type == CFG_ITEM_TYPE_MENU)
+		CFGItemSetValue(&list[i], (gconstpointer)value);
+
+	    return;
+	}
+	else
+	{
+	    /* Does not exist, are we allowed to create one? */
+/* TODO */
+	}
+}
+
