@@ -21,10 +21,10 @@
 #define PROG_NAME		"AVScan"
 #define PROG_NAME_FULL		"AntiVirus Scanner"
 
-#define PROG_VERSION		"1.3.1"
+#define PROG_VERSION		"3.2.1"
 
-#define PROG_VERSION_MAJOR	1
-#define PROG_VERSION_MINOR	3
+#define PROG_VERSION_MAJOR	3
+#define PROG_VERSION_MINOR	2
 #define PROG_VERSION_RELEASE	1
 
 /*
@@ -131,9 +131,10 @@ Usage: avscan <path(s)...> [options]\n\
 \n\
         --report-all-objects    Specifies to report all objects that\n\
                                 have been scanned to the Results List.\n\
-        --no-report-all-objects Specifies to not report all objects\n\
-                                that have been scanned to the Results\n\
-                                List.\n\
+        --no-report-all-objects Specifies to only report objects that\n\
+                                are infected or have problems to the\n\
+                                Results List (opposite of\n\
+                                --report-all-objects).\n\
 \n\
         --max-files <n>         Sets the maximum number of files in an\n\
                                 archive to scan.\n\
@@ -144,8 +145,8 @@ Usage: avscan <path(s)...> [options]\n\
         --max-compression-ratio <n>  Set the maximum archive\n\
                                 compression ratio to scan.\n\
 \n\
-        --help                  Prints (this) help screen and exits.\n\
-        --version               Prints version information and exits.\n\
+        --help                  Prints this help screen and exits.\n\
+        --version               Prints the version information and exits.\n\
 \n\
     Return values:\n\
 \n\
@@ -173,6 +174,8 @@ Usage: avscan <path(s)...> [options]\n\
         58      Reserved.\n\
         59      Scanner subsystem or plugin unable to scan an input\n\
                 file.\n\
+	60	Unsupported file format.\n\
+	61	Unable to lock the virus database directory.\n\
 \n\
         99      No virus scanner support compiled.\n\
 \n"
@@ -184,6 +187,7 @@ Usage: avscan <path(s)...> [options]\n\
 #define CFG_COMMENT_CHAR		'#'
 #define CFG_DELIMINATOR_CHAR		'='
 
+
 /*
  *	Environment Variable Names:
  */
@@ -191,6 +195,16 @@ Usage: avscan <path(s)...> [options]\n\
 #define ENV_VAR_NAME_HELPBROWSER	"HELPBROWSER"
 #define ENV_VAR_NAME_TMPDIR		"TMPDIR"
 
+
+/*
+ *	Widget Names:
+ */
+#define AVSCAN_SCANNER_TOPLEVEL_WIDGET_NAME	"avscan-scanner"
+#define AVSCAN_SCANNER_SCAN_CLIST_WIDGET_NAME	"avscan-scanner-scan-clist"
+#define AVSCAN_SCANNER_RESULTS_CLIST_WIDGET_NAME	"avscan-scanner-results-clist"
+#define AVSCAN_SCANNER_DB_CTREE_WIDGET_NAME	"avscan-scanner-db-ctree" 
+
+
 /*
  *	DND Infos:
  */
@@ -198,8 +212,9 @@ Usage: avscan <path(s)...> [options]\n\
 #define DND_INFO_TEXT_URI_LIST		1	/* "text/uri-list" */
 #define DND_INFO_STRING			2	/* "STRING" */
 
-#define DND_INFO_SCAN_ITEM		3	/* AVScan/ScanItem */
-#define DND_TARGET_SCAN_ITEM		"AVScan/ScanItem"
+#define AVSCAN_DND_INFO_SCAN_ITEM		3	/* AVScan/ScanItem */
+#define AVSCAN_DND_TARGET_SCAN_ITEM		"AVScan/ScanItem"
+
 
 /*
  *	List Defaults:
