@@ -10,7 +10,8 @@
 #include "win.h"
 
 
-typedef struct _core_struct     core_struct;
+typedef struct _core_struct		core_struct;
+#define CORE(p)				((core_struct *)(p))
 
 
 /*
@@ -26,6 +27,8 @@ struct _core_struct {
 	edv_context_struct	*edv_ctx;
 #endif
 
+	GdkAtom		target_scan_item_atom;
+
 	/* Pixmaps */
 	GdkPixmap	*scan_item_pixmap;
 	GdkBitmap	*scan_item_mask;
@@ -42,6 +45,8 @@ struct _core_struct {
 	GdkCursor	*busy_cur,
 			*passive_busy_cur;
 
+	guint		main_toid;
+
 	gboolean	close_all_windows;
 
 	/* Windows Lists */
@@ -58,7 +63,6 @@ struct _core_struct {
         gboolean	iconic;
 
 };
-#define CORE(p)		((core_struct *)(p))
 
 
 extern win_struct *MainNewAVScan(core_struct *core, gint argc, gchar **argv); 
