@@ -8,10 +8,13 @@
 #include <gtk/gtk.h>
 
 
+typedef struct _fb_type_struct		fb_type_struct;
+#define FB_TYPE(p)			((fb_type_struct *)(p))
+
+
 /*
  *	File Type:
  */
-typedef struct _fb_type_struct		fb_type_struct;
 struct _fb_type_struct {
 
 	gchar		*ext,		/* Space separated list of extensions */
@@ -23,6 +26,28 @@ struct _fb_type_struct {
 extern gint FileBrowserInit(void);
 extern void FileBrowserSetStyle(GtkRcStyle *rc_style);
 extern void FileBrowserSetTransientFor(GtkWidget *w);
+extern void FileBrowserSetObjectCreatedCB(
+	void (*cb)(
+		const gchar *path,
+		gpointer data
+	),
+	gpointer data
+);
+extern void FileBrowserSetObjectModifiedCB(
+	void (*cb)(
+		const gchar *old_path,
+		const gchar *new_path,
+		gpointer data
+	),
+	gpointer data
+);
+extern void FileBrowserSetObjectDeletedCB(
+	void (*cb)(
+		const gchar *path,
+		gpointer data
+	),
+	gpointer data
+);
 extern gboolean FileBrowserIsQuery(void);
 extern void FileBrowserBreakQuery(void);
 extern GtkWidget *FileBrowserGetToplevel(void);
@@ -39,17 +64,20 @@ extern void FileBrowserMap(void);
 extern void FileBrowserUnmap(void);
 extern void FileBrowserShutdown(void);
 
-extern void FileBrowserShowHiddenObjects(gboolean show);
+extern const gchar *FileBrowserGetLastLocation(void);
+extern void FileBrowserShowHiddenObjects(const gboolean show);
 extern void FileBrowserListStandard(void);
 extern void FileBrowserListDetailed(void);
 
+
+/* File Types */
 extern gint FileBrowserTypeListNew(
 	fb_type_struct ***list, gint *total,
 	const gchar *ext,
 	const gchar *name
 );
 extern void FileBrowserDeleteTypeList(
-	fb_type_struct **t, gint total
+	fb_type_struct **list, const gint total
 );
 
 
