@@ -12,44 +12,8 @@
 #include "images/icon_browse_20x20.xpm"
 
 
-/*
- *	Floating Prompt:
- */
 typedef struct _FPromptData		FPromptData;
-struct _FPromptData {
-
-	GtkWidget       *toplevel,	/* Popup GtkWindow */
-			*main_hbox,
-			*label,
-			*entry,
-			*browse_btn,
-			*ok_btn,
-			*cancel_btn,
-			*shadow;	/* Popup GtkWindow */
-	GdkPixmap	*shadow_pm;
-
-	gint		x, y,		/* Geometry of toplevel */
-			width, height;
-	fprompt_shadow	shadow_style;
-
-	gpointer	client_data;
-	gchar	*(*browse_cb)(
-		gpointer,		/* Data */
-		const gchar *		/* Value */
-	);
-	void	(*apply_cb)(
-		gpointer,		/* Data */
-		const gchar *		/* Value */
-	);
-	void	(*cancel_cb)(
-		gpointer		/* Data */
-	);
-
-};
-#define FPROMPT_DATA(p)	((FPromptData *)(p))
-
-
-static FPromptData		*fprompt = NULL;
+#define FPROMPT_DATA(p)			((FPromptData *)(p))
 
 
 static gint FPromptEntryEventCB(
@@ -65,23 +29,25 @@ static void FPromptCancelCB(GtkWidget *widget, gpointer data);
 static gint FPromptShadowPaintCB(FPromptData *p);
 static void FPromptShadowDraw(FPromptData *p);
 static void FPromptShadowConfigure(
-	FPromptData *p, gint x, gint y, gint width, gint height
+	FPromptData *p,
+	const gint x, const gint y,
+	const gint width, const gint height
 );
 
 gint FPromptInit(void);
 void FPromptSetTransientFor(GtkWidget *w);
 gboolean FPromptIsQuery(void);
 void FPromptBreakQuery(void);
-void FPromptSetPosition(gint x, gint y);
-void FPromptSetShadow(fprompt_shadow shadow);
+void FPromptSetPosition(const gint x, const gint y);
+void FPromptSetShadowStyle(const fprompt_shadow_style shadow);
 gint FPromptMapQuery(
 	const gchar *label,
 	const gchar *value,
 	const gchar *tooltip_message,
-	fprompt_map_code map_code,	/* One of FPROMPT_MAP_* */
-	gint width, gint height,
-	fprompt_flags flags,		/* Any of FPROMPT_FLAG_* */
-	gpointer client_data,
+	const fprompt_map_code map_code,
+	const gint width, const gint height,
+	const fprompt_flags flags,
+	gpointer data,
 	gchar *(*browse_cb)(gpointer, const gchar *),
 	void (*apply_cb)(gpointer, const gchar *),
 	void (*cancel_cb)(gpointer)
@@ -91,21 +57,66 @@ void FPromptUnmap(void);
 void FPromptShutdown(void);
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
-
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
-
 #define FPROMPT_SHADOW_OFFSET_X		5
 #define FPROMPT_SHADOW_OFFSET_Y		5
 
 
+/*
+ *	Floating Prompt:
+ */
+struct _FPromptData {
+
+	GtkWidget	*toplevel;	/* Popup GtkWindow */
+	gint		freeze_count;
+
+	GtkWidget	*main_hbox,
+			*label,
+			*entry,
+			*browse_btn,
+			*ok_btn,
+			*cancel_btn,
+			*shadow;	/* Shadow popup GtkWindow */
+
+	GdkPixmap	*shadow_pm;
+
+	gint		x, y,		/* Geometry of toplevel */
+			width, height;
+
+	fprompt_shadow_style	shadow_style;
+
+	gchar	*(*browse_cb)(
+		gpointer,		/* Data */
+		const gchar *		/* Value */
+	);
+	gpointer	browse_data;
+	void	(*apply_cb)(
+		gpointer,		/* Data */
+		const gchar *		/* Value */
+	);
+	gpointer	apply_data;
+	void	(*cancel_cb)(
+		gpointer		/* Data */
+	);
+	gpointer	cancel_data;
+
+};
+
+
+static FPromptData	*fprompt = NULL;
+
+
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
+
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
+
+
 /*
  *	Floating Prompt GtkEntry event signal callback.
  */
@@ -122,6 +133,9 @@ static gint FPromptEntryEventCB(
 	if((widget == NULL) || (event == NULL) || (p == NULL))
 	    return(status);
 
+	if(p->freeze_count > 0)
+	    return(status);
+
 	if(!FPromptIsQuery())
 	    return(status);
 
@@ -227,6 +241,9 @@ static gint FPromptEventCB(
 	if((widget == NULL) || (event == NULL) || (p == NULL))
 	    return(status);
 
+	if(p->freeze_count > 0)
+	    return(status);
+
 	if(!FPromptIsQuery())
 	    return(status);
 
@@ -266,6 +283,9 @@ static void FPromptBrowseCB(GtkWidget *widget, gpointer data)
 	if((widget == NULL) || (p == NULL))
 	    return;  
 
+	if(p->freeze_count > 0)
+	    return;
+
 	/* Browse callback set? */
 	if(p->browse_cb != NULL)
 	{
@@ -280,7 +300,7 @@ static void FPromptBrowseCB(GtkWidget *widget, gpointer data)
 	     * return string
 	     */
 	    const gchar *rtn_v = p->browse_cb(
-		p->client_data,		/* Data */
+		p->browse_data,		/* Data */
 		v			/* Value */
 	    );
 	    g_free(v);
@@ -300,6 +320,9 @@ static void FPromptOKCB(GtkWidget *widget, gpointer data)
 	if(p == NULL)
 	    return;
 
+	if(p->freeze_count > 0)
+	    return;
+
 	/* Apply callback set? */
 	if(p->apply_cb != NULL)
 	{
@@ -311,22 +334,25 @@ static void FPromptOKCB(GtkWidget *widget, gpointer data)
 	    {
 		/* Call apply callback */
 		p->apply_cb(
-		    p->client_data,	/* Data */
+		    p->apply_data,	/* Data */
 		    v			/* Value */
 		);
 		g_free(v);
 	    }
 	    p->apply_cb = NULL;
-	    p->client_data = NULL;
+	    p->apply_data = NULL;
 	}
 
-	/* Reset values */
-	p->client_data = NULL;
-	p->browse_cb = NULL; 
-	p->apply_cb = NULL;
-	p->cancel_cb = NULL;
-
+	/* Unmap the prompt */
 	FPromptUnmap();
+
+	/* Reset the callbacks */
+	p->browse_cb = NULL; 
+	p->browse_data = NULL;
+	p->apply_cb = NULL;
+	p->apply_data = NULL;
+	p->cancel_cb = NULL;
+	p->cancel_data = NULL;
 }
 
 /*
@@ -338,20 +364,25 @@ static void FPromptCancelCB(GtkWidget *widget, gpointer data)
 	if(p == NULL)
 	    return;
 
-	/* Cancel callback set? */
+	if(p->freeze_count > 0)
+	    return;
+
+	/* Call the cancel callback */
 	if(p->cancel_cb != NULL)
-	{
-	    /* Call cancel callback */
-	    p->cancel_cb(p->client_data);
-	}
-
-	/* Reset values */
-	p->client_data = NULL;
-	p->browse_cb = NULL; 
-	p->apply_cb = NULL;
-	p->cancel_cb = NULL;
+	    p->cancel_cb(
+		p->cancel_data			/* Data */
+	    );
 
+	/* Unmap the prompt */
 	FPromptUnmap();
+
+	/* Reset the callbacks */
+	p->browse_cb = NULL; 
+	p->browse_data = NULL;
+	p->apply_cb = NULL;
+	p->apply_data = NULL;
+	p->cancel_cb = NULL;
+	p->cancel_data = NULL;
 }
 
 
@@ -360,7 +391,14 @@ static void FPromptCancelCB(GtkWidget *widget, gpointer data)
  */
 static gint FPromptShadowPaintCB(FPromptData *p)
 {
+	if(p == NULL)
+	    return(FALSE);
+
+	if(p->freeze_count > 0)
+	    return(FALSE);
+
 	FPromptShadowDraw(p);
+
 	return(FALSE);
 }
 
@@ -398,24 +436,24 @@ static void FPromptShadowDraw(FPromptData *p)
  *	position.
  */
 static void FPromptShadowConfigure(
-	FPromptData *p, gint x, gint y, gint width, gint height
+	FPromptData *p,
+	const gint x, const gint y,
+	const gint width, const gint height
 )
 {
-	GtkWidget *w = (p != NULL) ? p->shadow : NULL;
-	if(w == NULL)
+	GtkWidget *w = p->shadow;
+	if((w == NULL) || (width <= 0) || (height <= 0))
 	    return;
 
-	if((width <= 0) || (height <= 0))
-	    return;
-
-	/* Update shadow geometry */
+	/* Update the shadow GtkWindow's geometry */
 	gtk_widget_set_uposition(
 	    w,
 	    x + FPROMPT_SHADOW_OFFSET_X,
 	    y + FPROMPT_SHADOW_OFFSET_Y
 	);
 	gtk_widget_set_usize(
-	    w, width, height
+	    w,
+	    width, height
 	);
 	gtk_widget_queue_resize(w);
 }
@@ -428,7 +466,7 @@ gint FPromptInit(void)
 {
 	const gint	border_minor = 2;
 	GdkWindow *window;
-	GtkWidget *w, *parent;
+	GtkWidget *w, *parent, *toplevel, *shadow;
 	FPromptData *p;
 
 	/* Already initialized */
@@ -440,25 +478,27 @@ gint FPromptInit(void)
 	if(p == NULL)
 	     return(-3);
 
-	/* Reset values */
+	p->toplevel = toplevel = gtk_window_new(GTK_WINDOW_POPUP);
+	p->freeze_count = 0;
+	p->shadow = shadow = gtk_window_new(GTK_WINDOW_POPUP);
 	p->shadow_pm = NULL;
 	p->x = 0;
 	p->y = 0;
 	p->width = FPROMPT_DEF_WIDTH;
 	p->height = FPROMPT_DEF_HEIGHT;
-	p->shadow_style = FPROMPT_SHADOW_SHADED;
-	p->client_data = NULL;
+	p->shadow_style = FPROMPT_SHADOW_DARKENED;
 	p->browse_cb = NULL;
+	p->browse_data = NULL;
 	p->apply_cb = NULL;
+	p->apply_data = NULL;
 	p->cancel_cb = NULL;
+	p->cancel_data = NULL;
 
-	/* Begin creating widgets */
+	p->freeze_count++;
 
-	/* Toplevel */
-	p->toplevel = w = gtk_window_new(GTK_WINDOW_POPUP);
-	gtk_window_set_policy(
-	    GTK_WINDOW(w), TRUE, TRUE, TRUE
-	);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_uposition(w, p->x, p->y);
 	gtk_widget_set_usize(w, p->width, p->height);
 	gtk_widget_realize(w);
@@ -584,8 +624,8 @@ gint FPromptInit(void)
 	);
 
 
-	/* Shadow */
-	p->shadow = w = gtk_window_new(GTK_WINDOW_POPUP);
+	/* Shadow GtkWindow */
+	w = shadow;
 	gtk_widget_set_uposition(w, p->x, p->y);
 	gtk_widget_set_usize(w, p->width, p->height);
 	gtk_widget_set_app_paintable(w, TRUE);
@@ -617,6 +657,8 @@ gint FPromptInit(void)
 
 	fprompt = p;
 
+	p->freeze_count--;
+
 	return(0);
 }
 
@@ -655,13 +697,24 @@ void FPromptBreakQuery(void)
 	 * cancel, reset values, and unmap the Floating Prompt
 	 */
 	if(FPromptIsQuery())
-	    FPromptCancelCB(NULL, p);
+	{
+	    /* Call the cancel callback */
+	    if(p->cancel_cb != NULL)
+		p->cancel_cb(
+		    p->cancel_data		/* Data */
+		);
 
-	/* Reset values */
-	p->client_data = NULL;
+	    /* Unmap the prompt */
+	    FPromptUnmap();
+	}
+
+	/* Reset the callbacks */
 	p->browse_cb = NULL;
+	p->browse_data = NULL;
 	p->apply_cb = NULL;
+	p->apply_data = NULL;
 	p->cancel_cb = NULL;
+	p->cancel_data = NULL;
 }
 
 /*
@@ -690,7 +743,7 @@ void FPromptSetPosition(gint x, gint y)
 /*
  *	Sets the Floating Prompt's shadow style.
  */
-void FPromptSetShadow(fprompt_shadow shadow)
+void FPromptSetShadowStyle(const fprompt_shadow_style shadow)
 {
 	FPromptData *p = fprompt;
 	if(p == NULL)
@@ -708,13 +761,13 @@ void FPromptSetShadow(fprompt_shadow shadow)
  *	Returns 0 on success or non-zero on error.
  */
 gint FPromptMapQuery(
-	const gchar *label,		/* Hides label if NULL */
+	const gchar *label,
 	const gchar *value,
 	const gchar *tooltip_message,
-	fprompt_map_code map_code,	/* One of FPROMPT_MAP_* */
-	gint width, gint height,
-	fprompt_flags flags,		/* Any of FPROMPT_FLAG_* */
-	gpointer client_data,
+	const fprompt_map_code map_code,
+	const gint width, const gint height,
+	const fprompt_flags flags,
+	gpointer data,
 	gchar *(*browse_cb)(gpointer, const gchar *),
 	void (*apply_cb)(gpointer, const gchar *),
 	void (*cancel_cb)(gpointer)
@@ -728,7 +781,7 @@ gint FPromptMapQuery(
 	toplevel = p->toplevel;
 	shadow = p->shadow;
 
-	/* Set new label */
+	/* Set the label */
 	w = p->label;
 	if(w != NULL)
 	{
@@ -743,13 +796,13 @@ gint FPromptMapQuery(
 	    }
 	}
 
-	/* Set new entry value */
+	/* Set the entry value */
 	w = p->entry;
 	if(w != NULL)
 	{
 	    if(value != NULL)
 	    {
-		gint len = STRLEN(value);
+		const gint len = STRLEN(value);
 		gtk_entry_set_text(GTK_ENTRY(w), value);
 		if(flags & FPROMPT_FLAG_SELECT_VALUE)
 		    gtk_entry_select_region(GTK_ENTRY(w), 0, len);
@@ -760,7 +813,7 @@ gint FPromptMapQuery(
 		/* Leave value it was */
 	    }
 
-	    /* Set new tooltip message */
+	    /* Set the tooltip message */
 	    GUISetWidgetTip(w, tooltip_message);
 	}
 
@@ -856,12 +909,16 @@ gint FPromptMapQuery(
 		break;
 	    }
 
-	    /* Create shadow pixmap */
-	    if(root != NULL)
+	    /* Recreate the shadow pixmap */
+	    if((p->shadow_style != FPROMPT_SHADOW_NONE) &&
+	       (shadow != NULL) && (root != NULL)
+	    )
 	    {
-		GtkWidget *w = p->shadow;
+		GtkWidget *w = shadow;
 		GdkWindow *window = w->window;
-		GdkPixmap *pixmap = GDK_PIXMAP_NEW(p->width, p->height);
+		GdkPixmap *pixmap = gdk_pixmap_new(
+		    window, p->width, p->height, -1
+		);
 		GtkStyle *style = gtk_widget_get_style(w);
 
 		GDK_PIXMAP_UNREF(p->shadow_pm);
@@ -871,10 +928,10 @@ gint FPromptMapQuery(
 		{
 		    gint mask_width = 8, mask_height = 2;
 		    guint8 bm_data_8x2[] = { 0x55, 0xaa };
+		    GdkBitmap *mask;
 		    GdkColor *c, shadow_color;
 		    GdkColormap *colormap = gdk_window_get_colormap(window);
 		    GdkGC *gc;
-		    GdkBitmap *mask;
 
 		    switch(p->shadow_style)
 		    {
@@ -882,8 +939,7 @@ gint FPromptMapQuery(
 			break;
 
 		      case FPROMPT_SHADOW_DITHERED:
-			gc = GDK_GC_NEW();
-
+			gc = gdk_gc_new(window);
 			gdk_gc_set_subwindow(gc, GDK_INCLUDE_INFERIORS);
 			gdk_window_copy_area(
 			    pixmap, gc,
@@ -896,7 +952,7 @@ gint FPromptMapQuery(
 			gdk_gc_set_subwindow(gc, GDK_CLIP_BY_CHILDREN);
 
 			mask = gdk_bitmap_create_from_data(
-			    root, (const gchar *)bm_data_8x2,
+			    window, (const gchar *)bm_data_8x2,
 			    mask_width, mask_height
 			);
 			if(mask != NULL)
@@ -917,20 +973,19 @@ gint FPromptMapQuery(
 			    }
 			    gdk_gc_set_clip_mask(gc, NULL);
 			}
-
 			GDK_GC_UNREF(gc);
 			GDK_BITMAP_UNREF(mask);
 			break;
 
-		      case FPROMPT_SHADOW_SOLID:
+		      case FPROMPT_SHADOW_BLACK:
 			gdk_draw_rectangle(
 			    pixmap, style->black_gc, TRUE,
 			    0, 0, p->width, p->height
 			);
 			break;
 
-		      case FPROMPT_SHADOW_SHADED:
-			gc = GDK_GC_NEW();
+		      case FPROMPT_SHADOW_DARKENED:
+			gc = gdk_gc_new(window);
 			c = &shadow_color;
 			c->red = 0x5fff;
 			c->green = 0x5fff;
@@ -962,15 +1017,22 @@ gint FPromptMapQuery(
 		    }
 		}
 	    }
+	    else
+	    {
+		GDK_PIXMAP_UNREF(p->shadow_pm);
+		p->shadow_pm = NULL;
+	    }
 	}
 
-	/* Set callbacks */
-	p->client_data = client_data;
-	p->browse_cb = browse_cb;
+	/* Set the callbacks */
+	p->browse_cb = browse_cb; 
+	p->browse_data = data;
 	p->apply_cb = apply_cb;
+	p->apply_data = data;
 	p->cancel_cb = cancel_cb;
+	p->cancel_data = data;
 
-	/* Map Floating Prompt */
+	/* Map the prompt */
 	FPromptMap();
 
 	return(0);
@@ -1037,12 +1099,6 @@ void FPromptShutdown(void)
 	    return;
 
 	GTK_WIDGET_DESTROY(p->shadow);
-	GTK_WIDGET_DESTROY(p->cancel_btn);
-	GTK_WIDGET_DESTROY(p->ok_btn);
-	GTK_WIDGET_DESTROY(p->browse_btn);
-	GTK_WIDGET_DESTROY(p->entry);
-	GTK_WIDGET_DESTROY(p->label);
-	GTK_WIDGET_DESTROY(p->main_hbox);
 	GTK_WIDGET_DESTROY(p->toplevel);
 
 	GDK_PIXMAP_UNREF(p->shadow_pm);
