@@ -29,7 +29,8 @@ static const gchar *TIME_DURATION_STRING(
 );
 
 #if defined(HAVE_CLAMAV)
-static gint WinDBGetPatternStatsFindIterate(
+# if defined(USE_CLAMAV_88)
+static gint WinDBGetPatternStatsFind88Iterate(
         win_struct *win,
 	const gchar *name,
         struct cl_node *clam_pnode,
@@ -38,6 +39,17 @@ static gint WinDBGetPatternStatsFindIterate(
         gulong *last_updated_time_rtn,
         gulong *discovery_time_rtn
 );
+# elif defined(USE_CLAMAV_90)
+static gint WinDBGetPatternStatsFind90Iterate(
+        win_struct *win,
+	const gchar *name,
+        struct cl_engine *clam_pnode,
+        guint8 **pattern_rtn,
+        gint *pattern_length_rtn,
+        gulong *last_updated_time_rtn,
+        gulong *discovery_time_rtn
+);
+# endif
 #endif
 gint WinDBGetPatternStats(
 	win_struct *win,
@@ -51,13 +63,23 @@ gint WinDBGetPatternStats(
 gint WinDBChanged(win_struct *win);
 
 #if defined(HAVE_CLAMAV)
-static void WinDBUpdateIterate(
+# if defined(USE_CLAMAV_88)
+static void WinDBUpdateClamAV88Iterate(
 	core_struct *core, win_struct *win,
 	GtkCTree *ctree, GtkCTreeNode *pnode, gchar **strv,
 	struct cl_node *clam_pnode,
 	const gulong start_time, const gboolean verbose,
 	gint *patterns_loaded, gulong *patterns_size_bytes
 );
+# else
+static void WinDBUpdateClamAV90Iterate(
+	core_struct *core, win_struct *win,
+	GtkCTree *ctree, GtkCTreeNode *pnode, gchar **strv,
+	struct cl_engine *clam_pnode,
+	const gulong start_time, const gboolean verbose,
+	gint *patterns_loaded, gulong *patterns_size_bytes
+);
+# endif
 #endif
 void WinDBUpdate(
 	win_struct *win, const gchar *db_location,
@@ -219,7 +241,8 @@ static const gchar *TIME_DURATION_STRING(
 
 
 #if defined(HAVE_CLAMAV)
-static gint WinDBGetPatternStatsFindIterate(
+# if defined(USE_CLAMAV_88)
+static gint WinDBGetPatternStatsFind88Iterate(
         win_struct *win,
 	const gchar *name,
         struct cl_node *clam_pnode,
@@ -244,7 +267,7 @@ static gint WinDBGetPatternStatsFindIterate(
 }
 
 	if(clam_pnode == NULL)
-	    return(1);
+	    return(-1);
 
 	/* Iterate through Extended Boyer-Moore list */
 	if(clam_pnode->bm_suffix != NULL)
@@ -480,10 +503,291 @@ static gint WinDBGetPatternStatsFindIterate(
 
 #undef PROCESS_FIND
 
-	return(1);
+	return(-1);
 }
+# elif defined(USE_CLAMAV_90)
+static gint WinDBGetPatternStatsFind90Iterate(
+        win_struct *win,
+	const gchar *name,
+        struct cl_engine *clam_pnode,
+        guint8 **pattern_rtn,
+        gint *pattern_length_rtn,
+        gulong *last_updated_time_rtn,
+        gulong *discovery_time_rtn
+)
+{
+#define PROCESS_FIND(_data_,_data_len_,_lut_,_dt_) {	\
+ if(pattern_rtn != NULL) {				\
+  if(((_data_) != NULL) && ((_data_len_) > 0))		\
+   *pattern_rtn = g_memdup((_data_), (_data_len_));	\
+ }							\
+ if(pattern_length_rtn != NULL)				\
+  *pattern_length_rtn = (_data_len_);			\
+							\
+ if(last_updated_time_rtn != NULL)			\
+  *last_updated_time_rtn = (_lut_);			\
+ if(discovery_time_rtn != NULL)				\
+  *discovery_time_rtn = (_dt_);				\
+}
+
+/* The CL_TARGET_TABLE_SIZE needs to be known for ClamAV 0.90.x
+ * in order to know the number of roots in the database
+ *
+ * Because this is not defined in the standard clamav.h header
+ * file we have taken it from clamav-0.90/libclamav/matcher.h
+ */
+#ifndef CL_TARGET_TABLE_SIZE
+# define CL_TARGET_TABLE_SIZE		7
 #endif
 
+	if(clam_pnode == NULL)
+	    return(-1);
+
+	/* Extended Boyer-Moore & Extended Aho-Corasick list */
+	if(clam_pnode->root != NULL)
+	{
+	    gint i;
+	    struct cli_matcher *matcher;
+
+	    for(i = 0; i < CL_TARGET_TABLE_SIZE; i++)
+	    {
+		matcher = clam_pnode->root[i];
+
+		/* Extended Boyer-Moore list */
+		if(!matcher->ac_only && (matcher->bm_suffix != NULL))
+		{
+		    struct cli_bm_patt *pattern;
+		    gint i;
+
+		    for(i = 0; matcher->bm_suffix[i] != NULL; i++)
+		    {
+			pattern = matcher->bm_suffix[i];
+			if(pattern->virname != NULL)
+			{
+			    if(!strcmp(pattern->virname, name))
+			    {
+				PROCESS_FIND(
+				    (const guint8 *)pattern->pattern,
+				    pattern->length,
+				    0l,
+				    0l
+				);
+				return(0);
+			    }
+			}
+			if(pattern->viralias != NULL)
+			{
+			    if(!strcmp(pattern->viralias, name))
+			    {
+				PROCESS_FIND(
+				    (const guint8 *)pattern->pattern,
+				    pattern->length,
+				    0l,
+				    0l
+				);
+				return(0);
+			    }
+			}
+		    }
+		}	/* Extended Boyer-Moore list */
+
+		/* Extended Aho-Corasick */
+		if(matcher->ac_nodes > 0)
+		{
+		    struct cli_ac_node *clam_ac_node;
+		    gint i;
+
+		    /* Iterate through Extended Aho-Corasick list */
+		    for(i = 0; i < matcher->ac_nodes; i++)
+		    {
+			/* Get this Extended Aho-Corasick node */
+			clam_ac_node = matcher->ac_nodetable[i];
+			if((clam_ac_node != NULL) ?
+			    (clam_ac_node->list != NULL) : FALSE
+			)
+			{
+			    /* Iterate through this list of patterns */
+			    struct cli_ac_patt *pattern;
+			    for(pattern = clam_ac_node->list;
+				pattern != NULL;
+					pattern = pattern->next
+			    )
+			    {
+				if(pattern->virname != NULL)
+				{
+				    if(!strcmp(pattern->virname, name))
+				    {
+					/* The pattern data is in 16 bits with
+					 * the high bit set to 0x00, need to
+					 * reduce/convert to 8 bit
+					 */
+					const gint len = (gint)pattern->length;
+					guint8 *p8 = (len > 0) ?
+					    (guint8 *)g_malloc(len) : NULL;
+					if(p8 != NULL)
+					{
+					    const guint8 *sp8 = (const guint8 *)pattern->pattern;
+					    guint8	*tp8 = p8,
+							*tpend8 = tp8 + len;
+					    while(tp8 < tpend8)
+					    {
+						*tp8 = *sp8;
+						tp8 += 1;
+						sp8 += 2;
+					    }
+					}
+				        PROCESS_FIND(
+					    p8,
+					    len,
+					    0l,
+					    0l
+					);
+					g_free(p8);
+					return(0);
+				    }
+				}
+				if(pattern->viralias != NULL)
+				{
+				    if(!strcmp(pattern->viralias, name))
+				    {
+					/* The pattern data is in 16 bits with
+					 * the high bit set to 0x00, need to
+					 * reduce/convert to 8 bit
+					 */
+					const gint len = (gint)pattern->length;
+					guint8 *p8 = (len > 0) ?
+					    (guint8 *)g_malloc(len) : NULL;
+					if(p8 != NULL)
+					{
+					    const guint8 *sp8 = (const guint8 *)pattern->pattern;
+					    guint8	*tp8 = p8,
+							*tpend8 = tp8 + len;
+					    while(tp8 < tpend8)
+					    {
+						*tp8 = *sp8;
+						tp8 += 1;
+						sp8 += 2;
+					    }
+					}
+				        PROCESS_FIND(
+					    p8,
+					    len,
+					    0l,
+					    0l
+					);
+					g_free(p8);
+					return(0);
+				    }
+				}
+			    }
+			}
+		    }
+		}	/* Extended Aho-Corasick */
+	    }
+	}
+
+	/* Iterate through MD5 list */
+	if(clam_pnode->md5_hlist != NULL)
+	{
+	    struct cli_md5_node *pattern;
+	    gint i;
+
+	    for(i = 0; clam_pnode->md5_hlist[i] != NULL; i++)
+	    {
+		for(pattern = clam_pnode->md5_hlist[i];
+		    pattern != NULL;
+		    pattern = pattern->next
+		)
+		{
+		    if(pattern->virname != NULL)
+		    {
+			if(!strcmp(pattern->virname, name))
+			{
+			    PROCESS_FIND(
+				(const guint8 *)pattern->md5,
+				pattern->size,
+				0l,
+				0l
+			    );
+			    return(0);
+			}
+		    }
+		    if(pattern->viralias != NULL)
+		    {
+			if(!strcmp(pattern->viralias, name))
+			{
+			    PROCESS_FIND(
+				(const guint8 *)pattern->md5,
+				pattern->size,
+				0l,
+				0l
+			    );
+			    return(0);
+			}
+		    }
+		}
+	    }
+	}
+
+	/* Iterate through ZIP metadata list */
+	if(clam_pnode->zip_mlist != NULL)
+	{
+	    struct cli_meta_node *pattern;
+
+	    for(pattern = clam_pnode->zip_mlist;
+		pattern != NULL;
+		pattern = pattern->next
+	    )
+	    {
+		if(pattern->virname != NULL)
+		{
+		    if(!strcmp(pattern->virname, name))
+		    {
+			PROCESS_FIND(
+			    (const guint8 *)pattern->filename,
+			    STRLEN(pattern->filename),
+			    0l,
+			    0l
+			);
+			return(0);
+		    }
+		}
+	    }
+	}
+
+	/* Iterate through RAR metadata list */
+	if(clam_pnode->rar_mlist != NULL)
+	{
+	    struct cli_meta_node *pattern;
+
+	    for(pattern = clam_pnode->rar_mlist;
+		pattern != NULL;
+		pattern = pattern->next
+	    )
+	    {
+		if(pattern->virname != NULL)
+		{
+		    if(!strcmp(pattern->virname, name))
+		    {
+			PROCESS_FIND(
+			    (const guint8 *)pattern->filename,
+			    STRLEN(pattern->filename),
+			    0l,
+			    0l
+			);
+			return(0);
+		    }
+		}
+	    }
+	}
+
+#undef PROCESS_FIND
+
+	return(-1);
+}
+# endif		/* USE_CLAMAV_90 */
+#endif		/* HAVE_CLAMAV */
+
 /*
  *	Gets the pattern statistics.
  *
@@ -511,7 +815,8 @@ gint WinDBGetPatternStats(
 	    return(-2);
 
 #if defined(HAVE_CLAMAV)
-	return(WinDBGetPatternStatsFindIterate(
+# if defined(USE_CLAMAV_88)
+	return(WinDBGetPatternStatsFind88Iterate(
 	    win, name,
 	    (struct cl_node *)win->db,
 	    pattern_rtn,
@@ -519,6 +824,18 @@ gint WinDBGetPatternStats(
 	    last_updated_time_rtn,
 	    discovery_time_rtn
 	));
+# elif defined(USE_CLAMAV_90)
+	return(WinDBGetPatternStatsFind90Iterate(
+	    win, name,
+	    (struct cl_engine *)win->db,
+	    pattern_rtn,
+	    pattern_length_rtn,
+	    last_updated_time_rtn,
+	    discovery_time_rtn
+	));
+# else
+	return(-1);
+# endif
 #else
 	return(-1);
 #endif
@@ -607,7 +924,25 @@ gint WinDBChanged(win_struct *win)
 }
 
 #if defined(HAVE_CLAMAV)
-static void WinDBUpdateIterate(
+# if defined(USE_CLAMAV_88)
+/*
+ *	Loads the nodes from the ClamAV node to the GtkCTreeNode.
+ *
+ *	The pnode specifies the parent GtkCTreeNode to load all
+ *	child nodes to.
+ *
+ *	The strv specifies the GtkCTree row values.
+ *
+ *	The clam_pnode specifies the ClamAV parent node to load all
+ *	child nodes from.
+ *
+ *	The patterns_loaded specifies the return for the total number
+ *	of patterns (pattern nodes, not parent nodes) loaded.
+ *
+ *	The patterns_size_bytes specifies the return for the total
+ *	size of the patterns loaded in bytes.
+ */
+static void WinDBUpdateClamAV88Iterate(
 	core_struct *core, win_struct *win,
 	GtkCTree *ctree, GtkCTreeNode *pnode, gchar **strv,
 	struct cl_node *clam_pnode,
@@ -1015,9 +1350,486 @@ static void WinDBUpdateIterate(
 	}
 
 #undef UPDATE_PATTERNS_LOADED_STATUS
-
-#endif
 }
+# else		/* USE_CLAMAV_88 */
+/*
+ *	Loads the nodes from the ClamAV node to the GtkCTreeNode.
+ *
+ * *********************************************************************
+ *
+ *	Important note, as of ClamAV 0.90.x the ClamAV virus database
+ *	patterns are not available to third party programs. The
+ *	following official statement was received by tkojm@clamav.net
+ *	on 2007-03-01:
+ *
+ *	"I do not see a point"
+ *
+ *	Which was in response to our request to have publically
+ *	available functions to access ClamAV virus patterns.
+ *
+ * *********************************************************************
+ *
+ *	The pnode specifies the parent GtkCTreeNode to load all
+ *	child nodes to.
+ *
+ *	The strv specifies the GtkCTree row values.
+ *
+ *	The clam_pnode specifies the ClamAV parent node to load all
+ *	child nodes from.
+ *
+ *	The patterns_loaded specifies the return for the total number
+ *	of patterns (pattern nodes, not parent nodes) loaded.
+ *
+ *	The patterns_size_bytes specifies the return for the total
+ *	size of the patterns loaded in bytes.
+ */
+static void WinDBUpdateClamAV90Iterate(
+	core_struct *core, win_struct *win,
+	GtkCTree *ctree, GtkCTreeNode *pnode, gchar **strv,
+	struct cl_engine *clam_pnode,
+	const gulong start_time, const gboolean verbose,
+	gint *patterns_loaded, gulong *patterns_size_bytes
+)
+{
+#if 0
+
+/* ClamAV nodes are now longer loaded, see note in above comment */
+
+	GtkCList *clist = GTK_CLIST(ctree);
+	const gint ncolumns = clist->columns;
+
+/* Increment *patterns_loaded, thaw/freeze the list, and update
+ * the status bar
+ */
+#define UPDATE_PATTERNS_LOADED_STATUS(_dpatterns_,_dpattern_size_) { \
+ /* Count this pattern as loaded */			\
+ *patterns_loaded = (*patterns_loaded) + (_dpatterns_);	\
+ *patterns_size_bytes = (*patterns_size_bytes) + (_dpattern_size_); \
+							\
+ /* Update progress every 50 patterns loaded */		\
+ if(verbose && (((*patterns_loaded) % 50) == 0)) {	\
+  WinStatusProgress(win, -1.0f, FALSE);			\
+  WinStatusTime(win, (gulong)time(NULL) - start_time, TRUE); \
+ }							\
+							\
+ /* Update status message every 100 patterns loaded */	\
+ if(verbose && (((*patterns_loaded) % 100) == 0)) {	\
+  gchar *s2 = STRDUP(EDVSizeStrDelim(			\
+    (gulong)(*patterns_loaded)				\
+  ));							\
+  gchar *s = g_strdup_printf(				\
+   "%s patterns loaded...",				\
+   s2							\
+  );							\
+  g_free(s2);						\
+  WinStatusMessage(win, s, TRUE);			\
+  g_free(s);						\
+ }							\
+							\
+ /* Thaw and refreeze list every 500 patterns loaded */	\
+ if(((*patterns_loaded) % 500) == 0) {			\
+  gtk_clist_thaw(clist);				\
+  gtk_clist_freeze(clist);				\
+ }							\
+}
+
+/* The CL_TARGET_TABLE_SIZE needs to be known for ClamAV 0.90.x
+ * in order to know the number of roots in the database
+ *
+ * Because this is not defined in the standard clamav.h header
+ * file we have taken it from clamav-0.90/libclamav/matcher.h
+ */
+#ifndef CL_TARGET_TABLE_SIZE
+# define CL_TARGET_TABLE_SIZE		7
+#endif
+
+	/* Extended Boyer-Moore & Extended Aho-Corasick list */
+	if((clam_pnode->root != NULL) && (win->stop_count == 0))
+	{
+	    gint i;
+	    struct cli_matcher *matcher;
+
+	    for(i = 0; i < CL_TARGET_TABLE_SIZE; i++)
+	    {
+		if(win->stop_count > 0)
+		    break;
+
+		matcher = clam_pnode->root[i];
+
+		/* Extended Boyer-Moore list */
+		if(!matcher->ac_only && (matcher->bm_suffix != NULL))
+		{
+		    struct cli_bm_patt *pattern;
+		    gint i;
+		    gchar *pattern_label;
+		    GtkCTreeNode *node, *pnode2;
+
+		    for(i = 0; matcher->bm_suffix[i] != NULL; i++)
+		    {
+			if(win->stop_count > 0)
+			    break;
+
+			/* Get this list of patterns */
+		        pattern = matcher->bm_suffix[i];
+
+			INSERT_NODE_FOLDER(
+			    "Extended Boyer-Moore", pnode, NULL
+			);
+			if(node == NULL)
+			    break;
+
+			pnode2 = node;
+
+			/* Iterate through this list of patterns */
+			while(pattern != NULL)
+			{
+			    if(win->stop_count > 0)
+				break;
+
+			    if(!STRISEMPTY(pattern->virname) &&
+			       !STRISEMPTY(pattern->viralias)
+			    )
+			    {
+				pattern_label = g_strdup_printf(
+				    "%s (%s)",
+				    pattern->virname,
+				    pattern->viralias
+				);
+			    }     
+			    else if(!STRISEMPTY(pattern->virname))
+			    {
+				pattern_label = STRDUP(pattern->virname);
+			    }
+			    else if(!STRISEMPTY(pattern->viralias))
+			    {
+				pattern_label = STRDUP(pattern->viralias);
+			    }
+			    else
+			    {
+				pattern = pattern->next;
+				continue;
+			    }
+
+			    /* Add this pattern to the list */
+			    INSERT_NODE_ITEM(
+				pattern_label, pnode2, NULL
+			    );
+			    g_free(pattern_label);
+
+			    UPDATE_PATTERNS_LOADED_STATUS(1, pattern->length);
+
+			    /* Get the next pattern */
+			    pattern = pattern->next;
+			}
+		    }
+		}	/* Extended Boyer-Moore */
+
+		if(win->stop_count > 0)
+		    break;
+
+		/* Extended Aho-Corasick */
+		if(matcher->ac_nodes > 0)
+		{
+		    struct cli_ac_node *clam_ac_node;
+		    gint i;
+		    GtkCTreeNode *node, *pnode2;
+
+		    INSERT_NODE_FOLDER(
+			"Extended Aho-Corasick", pnode, NULL
+		    );
+		    pnode2 = node;
+
+		    /* Iterate through Extended Aho-Corasick list */
+		    for(i = 0; i < matcher->ac_nodes; i++)
+		    {
+			if(win->stop_count > 0)
+			    break;
+
+			/* Get this Extended Aho-Corasick node */
+			clam_ac_node = matcher->ac_nodetable[i];
+			if((clam_ac_node != NULL) ?
+			    (clam_ac_node->list != NULL) : FALSE
+			)
+			{
+			    gchar *pattern_label;
+
+			    /* Get this list of patterns */
+			    struct cli_ac_patt *pattern = clam_ac_node->list;
+
+			    GtkCTreeNode *pnode3;
+
+			    /* Create a subfolder if this node has multiple
+			     * patterns
+			     */
+			    if(pattern->next != NULL)
+			    {
+				if(!STRISEMPTY(pattern->virname) &&
+				   !STRISEMPTY(pattern->viralias)  
+				)
+				{
+				    pattern_label = g_strdup_printf(
+					"%s (%s)",
+					pattern->virname,
+					pattern->viralias
+				    );
+				}
+				else if(!STRISEMPTY(pattern->virname))
+				{
+				    pattern_label = STRDUP(pattern->virname);
+				}
+				else if(!STRISEMPTY(pattern->viralias))
+				{
+				    pattern_label = STRDUP(pattern->viralias);
+				}
+				else
+				{
+				    pattern_label = STRDUP("");
+				}
+				INSERT_NODE_FOLDER(
+				    pattern_label, pnode2, NULL
+				);
+				g_free(pattern_label);
+				pnode3 = node;
+			    }
+			    else
+			    {
+				pnode3 = pnode2;
+			    }
+
+			    /* Iterate through this list of patterns */
+			    while(pattern != NULL)
+			    {
+				if(win->stop_count > 0)
+				    break;
+	
+				if(!STRISEMPTY(pattern->virname) && 
+				   !STRISEMPTY(pattern->viralias)
+				)
+				{
+				    pattern_label = g_strdup_printf(
+					"%s (%s)",
+					pattern->virname,
+					pattern->viralias
+				    );
+				}
+				else if(!STRISEMPTY(pattern->virname))
+				{
+				    pattern_label = STRDUP(pattern->virname);
+				}
+				else if(!STRISEMPTY(pattern->viralias))
+				{
+				    pattern_label = STRDUP(pattern->viralias);
+				}
+				else
+				{
+				    pattern = pattern->next;
+				    continue;
+				}   
+
+				/* Add this pattern to the list */
+				INSERT_NODE_ITEM(
+				    pattern_label, pnode3, NULL
+				);
+				g_free(pattern_label);
+
+				UPDATE_PATTERNS_LOADED_STATUS(1, pattern->length);
+
+				/* Get the next pattern */
+				pattern = pattern->next;
+			    }
+			}
+		    }
+		}	/* Extended Aho-Corasick */
+
+	    }
+	}
+
+
+	/* MD5 list */
+	if((clam_pnode->md5_hlist != NULL) && (win->stop_count == 0))
+	{
+	    struct cli_md5_node *pattern;
+	    gint i;
+	    gchar *pattern_label;
+	    GtkCTreeNode *node, *pnode2;
+
+	    for(i = 0; clam_pnode->md5_hlist[i] != NULL; i++)
+	    {
+		if(win->stop_count > 0)
+		    break;
+
+		/* Get this list of patterns */
+		pattern = clam_pnode->md5_hlist[i];
+
+		INSERT_NODE_FOLDER(
+		    "MD5", pnode, NULL
+		);
+		if(node == NULL)
+		    break;
+
+		pnode2 = node;
+
+		/* Iterate through this list of patterns */
+		while(pattern != NULL)
+		{
+		    if(win->stop_count > 0)
+			break;
+
+		    if(!STRISEMPTY(pattern->virname) &&
+		       !STRISEMPTY(pattern->viralias) 
+		    )
+		    {
+			pattern_label = g_strdup_printf(
+			    "%s (%s)",
+			    pattern->virname,
+			    pattern->viralias
+			);
+		    }
+		    else if(!STRISEMPTY(pattern->virname))
+		    {
+			pattern_label = STRDUP(pattern->virname);
+		    }
+		    else if(!STRISEMPTY(pattern->viralias))
+		    {
+			pattern_label = STRDUP(pattern->viralias);
+		    }
+		    else
+		    {
+			pattern = pattern->next;
+			continue;
+		    }
+
+		    /* Add this pattern to the list */
+		    INSERT_NODE_ITEM(
+			pattern_label, pnode2, NULL
+		    );
+		    g_free(pattern_label);
+
+		    UPDATE_PATTERNS_LOADED_STATUS(1, pattern->size);
+
+		    /* Get the next pattern */
+		    pattern = pattern->next;
+		}
+	    }
+	}
+
+	/* Iterate through ZIP metadata list */
+	if((clam_pnode->zip_mlist != NULL) && (win->stop_count == 0))
+	{
+	    GtkCTreeNode *node;
+
+	    INSERT_NODE_FOLDER(
+		"ZIP metadata", pnode, NULL
+	    );
+	    if(node != NULL)
+	    {
+		gchar *pattern_label;
+		struct cli_meta_node *pattern;
+		GtkCTreeNode *pnode2 = node;
+
+		for(pattern = clam_pnode->zip_mlist;
+		    pattern != NULL;
+		    pattern = pattern->next
+		)
+		{
+		    if(win->stop_count > 0)
+			break;
+
+		    if(!STRISEMPTY(pattern->virname) &&
+		       !STRISEMPTY(pattern->filename)
+		    )
+		    {
+			pattern_label = g_strdup_printf(
+			    "%s - %s",
+			    pattern->virname,
+			    pattern->filename
+			);
+		    }
+		    else if(!STRISEMPTY(pattern->virname))
+		    {
+			pattern_label = STRDUP(pattern->virname);
+		    }
+		    else if(!STRISEMPTY(pattern->filename))
+		    {
+			pattern_label = STRDUP(pattern->filename);
+		    }
+		    else
+		    {
+			continue;
+		    }
+
+		    /* Add this pattern to the list */
+		    INSERT_NODE_ITEM(
+			pattern_label, pnode2, NULL
+		    );
+		    g_free(pattern_label);
+
+		    UPDATE_PATTERNS_LOADED_STATUS(1, pattern->size);
+		}
+	    }
+	}
+
+	/* Iterate through RAR metadata list */
+	if((clam_pnode->rar_mlist != NULL) && (win->stop_count == 0))
+	{
+	    GtkCTreeNode *node;
+
+	    INSERT_NODE_FOLDER(
+		"RAR metadata", pnode, NULL
+	    );
+	    if(node != NULL)
+	    {
+		gchar *pattern_label;
+		struct cli_meta_node *pattern;
+		GtkCTreeNode *pnode2 = node;
+
+		for(pattern = clam_pnode->rar_mlist;
+		    pattern != NULL;
+		    pattern = pattern->next
+		)
+		{
+		    if(win->stop_count > 0)
+			break;
+
+		    if(!STRISEMPTY(pattern->virname) &&
+		       !STRISEMPTY(pattern->filename)
+		    )
+		    {
+			pattern_label = g_strdup_printf(
+			    "%s - %s",
+			    pattern->virname,
+			    pattern->filename
+			);
+		    }
+		    else if(!STRISEMPTY(pattern->virname))
+		    {
+			pattern_label = STRDUP(pattern->virname);
+		    }
+		    else if(!STRISEMPTY(pattern->filename))
+		    {
+			pattern_label = STRDUP(pattern->filename);
+		    }
+		    else
+		    {
+			continue;
+		    }
+
+		    /* Add this pattern to the list */
+		    INSERT_NODE_ITEM(
+			pattern_label, pnode2, NULL
+		    );
+		    g_free(pattern_label);
+
+		    UPDATE_PATTERNS_LOADED_STATUS(1, pattern->size);
+		}
+	    }
+	}
+
+#undef UPDATE_PATTERNS_LOADED_STATUS
+#endif	/* #if 0 */
+}
+# endif
+#endif		/* HAVE_CLAMAV */
 
 /*
  *	Updates the Win's Virus Database Tree with the information
@@ -1030,7 +1842,11 @@ void WinDBUpdate(
 {
 #if defined(HAVE_CLAMAV)
 	struct stat stat_buf;
+# if defined(USE_CLAMAV_88)
 	struct cl_node *database = NULL;
+# else
+	struct cl_engine *database = NULL;
+# endif
 	int database_entries = 0;
 	gint status, patterns_loaded = 0;
 	gulong patterns_total_size_bytes = 0l;
@@ -1058,7 +1874,6 @@ void WinDBUpdate(
 	 */
 	WinDBClear(win);
 
-
 	/* Begin updating */
 
 	/* Reset the stop count and update so the stop button can
@@ -1136,19 +1951,13 @@ void WinDBUpdate(
 	    const gint error_code = (gint)errno;
 	    if(verbose)
 	    {
-		gchar *s, *error_msg = STRDUP(g_strerror(error_code));
-		if(error_msg != NULL)
-		{
-		    *error_msg = (gchar)toupper((int)*error_msg);
-		    s = g_strdup_printf(
-			"%s: %s",
-			db_location,
-			error_msg
-		    );
-		    WinStatusMessage(win, s, FALSE);
-		    g_free(s);
-		    g_free(error_msg);
-		}
+		gchar *msg = g_strdup_printf(
+"%s: %s.",
+                    db_location,
+                    g_strerror(error_code)
+		);
+		WinStatusMessage(win, msg, FALSE);
+		g_free(msg);
 		WinStatusProgress(win, 0.0f, FALSE);
 		WinStatusTime(win, 0l, FALSE);
 	    }
@@ -1210,10 +2019,24 @@ void WinDBUpdate(
 		    }
 		}
 		start_time = (gulong)time(NULL);
+#if defined(USE_CLAMAV_88)
 		status = cl_loaddbdir(
 		    db_location,
 		    &database, &database_entries
 		);
+#else	/* ClamAV 0.90.x and up */
+		/* Do not load for ClamAV 0.90.x, see comment above
+		 * function WinDBUpdateClamAV90Iterate()
+		 */
+		status = 9;
+# if 0
+		status = cl_load(
+		    db_location,
+		    &database, &database_entries,
+		    CL_DB_STDOPT
+		);
+# endif
+#endif
 	    }
 	    else
 	    {
@@ -1229,10 +2052,24 @@ void WinDBUpdate(
 		    g_free(parent);
 		}
 		start_time = (gulong)time(NULL);
+#if defined(USE_CLAMAV_88)
 		status = cl_loaddb(
 		    db_location,
 		    &database, &database_entries
 		);
+#else	/* ClamAV 0.90.x and up */
+		/* Do not load for ClamAV 0.90.x, see comment above
+		 * function WinDBUpdateClamAV90Iterate()
+		 */
+		status = 9;
+# if 0
+		status = cl_load(
+		    db_location,
+		    &database, &database_entries,
+		    CL_DB_STDOPT
+		);
+# endif
+#endif
 	    }
 	}
 	else
@@ -1265,13 +2102,13 @@ void WinDBUpdate(
 	    {
 		if(verbose)
 		{
-		    gchar *s = g_strconcat(
+		    gchar *msg = g_strconcat(
 			"Virus database build error: ",
 			cl_strerror(status),
 			NULL
 		    );
-		    WinStatusMessage(win, s, FALSE);
-		    g_free(s);
+		    WinStatusMessage(win, msg, FALSE);
+		    g_free(msg);
 		    WinStatusProgress(win, 0.0f, FALSE);
 		    WinStatusTime(win, 0l, FALSE);
 		}
@@ -1317,12 +2154,24 @@ void WinDBUpdate(
 	    win->db_root_node = node;
 
 	    /* Add all child nodes and items to this node */
-	    WinDBUpdateIterate(
+#if defined(USE_CLAMAV_88)
+	    WinDBUpdateClamAV88Iterate(
 		core, win, ctree, node, strv,
-		database,
+		database,		/* Root node */
 		start_time, verbose,
 		&patterns_loaded, &patterns_total_size_bytes
 	    );
+#else	/* ClamAV 0.90.x and up */
+	    /* Does nothing, ClamAV 0.90.x does not have publically
+	     * available virus patterns
+	     */
+	    WinDBUpdateClamAV90Iterate(
+		core, win, ctree, node, strv,
+		database,		/* Root node */
+		start_time, verbose,
+		&patterns_loaded, &patterns_total_size_bytes
+	    );
+#endif
 
 	    gtk_clist_thaw(clist);
 	    if(verbose)
@@ -1333,14 +2182,15 @@ void WinDBUpdate(
 	/* Unable to load the virus database? */
 	else if(win->stop_count == 0)
 	{
-	    gchar *s = g_strconcat(
+#if defined(USE_CLAMAV_88)
+	    gchar *msg = g_strconcat(
 		"Virus database load error: ",
 		cl_strerror(status),
 		NULL
 	    );
-	    WinStatusMessage(win, s, FALSE);
-	    g_free(s);
-
+	    WinStatusMessage(win, msg, FALSE);
+	    g_free(msg);
+#endif
 	    /* Delete the virus database */
 	    if(database != NULL)
 		cl_free(database);
@@ -1405,6 +2255,7 @@ void WinDBUpdate(
 		    (gint)(duration_t % 60l)
 		);
 
+#if defined(USE_CLAMAV_88)
 	    msg = g_strdup_printf(
 		"%s\n%s\n%s (%s) %s%s",
 		db_location_s,
@@ -1414,6 +2265,19 @@ void WinDBUpdate(
 		duration_s,
 		(win->stop_count > 0) ? " (Interrupted)" : " load time"
 	    );
+#elif defined(USE_CLAMAV_90)
+	    msg = g_strdup_printf(
+		"%s\n%s\nClamAV 0.90.x patterns are private",
+		db_location_s,
+		last_updated_s
+	    );
+#else
+	    msg = g_strdup_printf(
+		"%s\n%s\nClamAV 0.91.x patterns are private",
+		db_location_s,
+		last_updated_s
+	    );
+#endif
 	    gtk_label_set_text(GTK_LABEL(w), msg);
 	    g_free(msg);
 	    g_free(db_location_s);
@@ -1429,12 +2293,17 @@ void WinDBUpdate(
 	    gchar *patterns_loaded_s = STRDUP(EDVSizeStrDelim(
 		(gulong)patterns_loaded
 	    ));
+#if defined(USE_CLAMAV_88)
 	    gchar *msg = g_strdup_printf(
 		"Virus database loaded with %s patterns",
 		patterns_loaded_s
 	    );
-
-	    /* Raise Win */
+#else
+	    gchar *msg = STRDUP(
+		"Virus database loaded"
+	    );
+#endif
+	    /* Raise the Win */
 	    WinMap(win);
 
 	    WinStatusMessage(win, msg, FALSE);
