@@ -1,4 +1,5 @@
 #include <string.h>
+#include <time.h>
 #include <gtk/gtk.h>
 #include "guiutils.h"
 #include "cdialog.h"
@@ -81,9 +82,8 @@ static void WinDragDataReceivedNexus(
 	/* Animation Icon */
 	if(w == win->anim_icon)
 	{
-	    gchar *location;
+	    GList *paths_list = NULL;
 	    GtkWidget *toplevel = win->toplevel;
-	    GtkEntry *entry = GTK_ENTRY(win->location_entry);
 
 	    /* Is there a scan or update already in progress? */
 	    if((win->scan_context->toid != 0) ||
@@ -121,14 +121,13 @@ current scan or update before starting a new scan.",
 	       (info == DND_INFO_STRING)
 	    )
 	    {
-		gchar *s;
+		const gchar *path;
 		GList *glist, *obj_list = ObjDDEBufferParsePath(
 		    (const guint8 *)selection_data->data,
 		    selection_data->length
 		);
 		obj_struct *obj;
 
-		location = STRDUP("");
 		for(glist = obj_list;
 		    glist != NULL;
 		    glist = g_list_next(glist)
@@ -138,30 +137,32 @@ current scan or update before starting a new scan.",
 		    if(obj == NULL)
 			continue;
 
-		    s = g_strconcat(
-			location,
-			obj->path,
-			(g_list_next(glist) != NULL) ? "," : "",
-			NULL
+		    path = obj->path;
+		    if(STRISEMPTY(path))
+			continue;
+
+		    paths_list = g_list_append(
+			paths_list,
+			STRDUP(path)
 		    );
-		    g_free(location);
-		    location = s;
 
 		    ObjDelete(obj);
 		}
 		g_list_free(obj_list);
 	    }
 	    /* Scan Item */
-	    else if(info == DND_INFO_SCAN_ITEM)
+	    else if(info == AVSCAN_DND_INFO_SCAN_ITEM)
 	    {
-		gchar *s;
+		const gulong start_time = (gulong)time(NULL);
+		gint i;
+		gchar **strv;
+		const gchar *location;
 		GList *glist, *obj_list = ObjDDEBufferParse(
 		    (const guint8 *)selection_data->data,
 		    selection_data->length
 		);
 		obj_struct *obj;
 
-		location = STRDUP("");
 		for(glist = obj_list;
 		    glist != NULL;
 		    glist = g_list_next(glist)
@@ -171,14 +172,37 @@ current scan or update before starting a new scan.",
 		    if(obj == NULL)
 			continue;
 
-		    s = g_strconcat(
+		    location = obj->path;
+		    if(STRISEMPTY(location))
+			continue;
+
+		    /* Update the last scanned time of all the
+		     * scan items who's location matches this
+		     * location
+		     */
+		    WinScanListTouchByLocation(
+			win,
 			location,
-			obj->path,
-			(g_list_next(glist) != NULL) ? "," : "",
-			NULL
+			start_time
 		    );
-		    g_free(location);
-		    location = s;
+
+		    /* Need to explode the location so we get
+		     * each path
+		     */
+		    strv = g_strsplit(location, ",", -1);
+		    if(strv == NULL)
+			continue;
+
+		    /* Append each path from the location to the
+		     * paths list
+		     */
+		    for(i = 0; strv[i] != NULL; i++)
+			paths_list = g_list_append(
+			    paths_list,
+			    STRDUP(strv[i])
+			);
+
+		    g_strfreev(strv);
 
 		    ObjDelete(obj);
 		}
@@ -215,23 +239,28 @@ is not supported by this application.",
 		);
 		CDialogSetTransientFor(NULL);
 		g_free(msg);
-
-		location = NULL;
 	    }
-	    if(location == NULL)
+	    if(paths_list == NULL)
 	    {
 		WinSetBusy(win, FALSE);
 		return;
 	    }
 
-	    /* Set the location of the scan */
-	    gtk_entry_set_text(entry, location);
-	    gtk_entry_set_position(entry, -1);
-
-	    g_free(location);
-
 	    /* Start the scan */
-	    WinStartCB(NULL, win);
+	    WinScanProcessStart(
+		win,
+		NULL,			/* Use the virus database path
+					 * specified in the configuration */
+		paths_list,
+		WinGetCurrentOptions(win)
+	    );
+
+	    /* Switch to the results page */
+	    WinSwitchPage(win, WIN_PAGE_NUM_RESULTS);
+
+	    /* Delete the paths list */
+	    g_list_foreach(paths_list, (GFunc)g_free, NULL);
+	    g_list_free(paths_list);
 	}
 	/* Scan List? */
 	else if(w == win->scan_clist)
@@ -255,7 +284,7 @@ is not supported by this application.",
 		);
 	    }
 	    /* Scan Item */
-	    else if(info == DND_INFO_SCAN_ITEM)
+	    else if(info == AVSCAN_DND_INFO_SCAN_ITEM)
 	    {
 		obj_list = ObjDDEBufferParse(
 		    (const guint8 *)selection_data->data,
@@ -350,7 +379,7 @@ is not supported by this application.",
 	    gtk_clist_thaw(clist);
 	}
 	/* Location Entry */
-	else if(w == win->location_entry)
+	else if(w == win->scan_location_entry)
 	{
 	    gchar *location;
 	    GtkEntry *entry = GTK_ENTRY(w);
@@ -396,7 +425,7 @@ is not supported by this application.",
 		g_list_free(obj_list);
 	    }
 	    /* Scan Item */
-	    else if(info == DND_INFO_SCAN_ITEM)
+	    else if(info == AVSCAN_DND_INFO_SCAN_ITEM)
 	    {
 		gchar *s;
 		GList *glist, *obj_list = ObjDDEBufferParse(
