@@ -35,6 +35,11 @@ void WinScanListUpdateRow(
 	win_struct *win, const gint row,
 	const obj_struct *obj
 );
+void WinScanListTouchByLocation(
+        win_struct *win,
+	const gchar *location,
+	const gulong t
+);
 
 gint WinResultsListAppend(
 	win_struct *win,
@@ -44,16 +49,16 @@ gint WinResultsListAppend(
 void WinResultsListClear(win_struct *win);
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
 
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
 /*
@@ -149,7 +154,7 @@ static const gchar *TIME_DURATION_STRING(
 	    ct = MAX(dt / 60 / 60 / 24 / 30, 1);
 	    g_snprintf(
 		buf, sizeof(buf),
-		"%ld month%s",
+		"%ld month%s ago",
 		ct, (ct == 1) ? "" : "s"
 	    );
 	}
@@ -231,7 +236,7 @@ gint WinScanListInsert(
 	    /* Update the row cell values with the new object */
 	    WinScanListUpdateRow(win, new_row, obj);
 
-	    /* Set the Object as the row data */
+	    /* Set the new row's data as the Object */
 	    gtk_clist_set_row_data_full(
 		clist, new_row,
 		obj, WinScanListItemDestroyCB
@@ -494,9 +499,6 @@ void WinScanListUpdateRow(
 
 	g_free(path);
 
-	/* Resize the columns */
-	gtk_clist_columns_autosize(clist);
-
 	gtk_clist_thaw(clist);
 
 	GDK_PIXMAP_UNREF(name_pixmap);
@@ -505,6 +507,44 @@ void WinScanListUpdateRow(
 	GDK_BITMAP_UNREF(location_mask);
 }
 
+/*
+ *	Updates the last runned time of all scan items who's
+ *	location matches the specified location.
+ */
+void WinScanListTouchByLocation(
+        win_struct *win,
+	const gchar *location,
+	const gulong t
+)
+{
+	gint i, n;
+	GtkCList *clist;
+	obj_struct *obj;
+
+	if((win == NULL) || STRISEMPTY(location))
+	    return;
+
+	clist = GTK_CLIST(win->scan_clist);
+        n = clist->rows;
+        for(i = 0; i < n; i++)
+        {
+            obj = OBJ(gtk_clist_get_row_data(clist, i));
+            if(obj == NULL)
+                continue;
+
+            if(obj->path == NULL)
+                continue;
+
+            /* Object's location matches the specified location? */
+            if(!strcmp((const char *)obj->path, (const char *)location))
+            {
+                /* Update this object's last runned time */
+                obj->last_runned = t;
+                WinScanListUpdateRow(win, i, obj);
+            }
+        }
+}
+
 
 /*
  *	Appends the row to the Win's Results List.
@@ -569,6 +609,7 @@ gint WinResultsListAppend(
 	    if(infected)
 		obj->options |= AVSCAN_OPT_INFECTED;
 
+	    /* Set the new row's data as the Object */
 	    gtk_clist_set_row_data_full(
 		clist, new_row,
 		obj, WinResultsListItemDestroyCB
@@ -601,7 +642,7 @@ gint WinResultsListAppend(
 		);
 	    }     
 	    /* Match MIME Type */
-	    else if(!lstat(path, &lstat_buf))
+	    else if(!lstat((const char *)path, &lstat_buf))
 	    {
 		edv_mime_type_struct *m = EDVMimeTypeMatch(
 		    ctx, path, &lstat_buf
@@ -653,7 +694,6 @@ gint WinResultsListAppend(
 	    );
 	}
 
-	/* Resize columns */
 	gtk_clist_columns_autosize(clist);
 
 	gtk_clist_thaw(clist);
@@ -670,12 +710,14 @@ gint WinResultsListAppend(
  */
 void WinResultsListClear(win_struct *win)
 {
-	GtkCList *clist = (win != NULL) ?
-	    (GtkCList *)win->results_clist : NULL;
-	if(clist == NULL)
+	GtkCList *clist;
+
+	if(win == NULL)
 	    return;
 
+	clist = GTK_CLIST(win->results_clist);
 	gtk_clist_freeze(clist);
 	gtk_clist_clear(clist);
+	gtk_clist_columns_autosize(clist);
 	gtk_clist_thaw(clist);
 } 
